 // Components for manipulating sequences of characters -*- C++ -*-

// Copyright  1997-2015 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or 
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Under Section 7 of GPL version 3, you are granted additional
// permissions described in the GCC Runtime Library Exception, version
// 3.1, as published by the Free Software Foundation.

// You should have received a copy of the GNU General Public License and
// a copy of the GCC Runtime Library Exception along with this program;
// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
// .

/** @file bits/basic_string.tcc
 *  This is an internal header file, included by other library headers.
 *  Do not attempt to use it directly. @headername
 */

//
// ISO C++ 14882: 21  Strings library
//

// Written by Jason Merrill based upon the specification by Takanori Adachi
// in ANSI X3J16/94-0013R2.  Rewritten by Nathan Myers to ISO-14882.
// Non-reference-counted implementation written by Paolo Carlini and
// updated by Jonathan Wakely for ISO-14882-2011.

#ifndef _BASIC_STRING_TCC
#define _BASIC_STRING_TCC 1

#pragma GCC system_header

#pragma "/home/regehr/z/compiler-install/gcc-r231221-install/include/c++/6.0.0/bits/cxxabi_forced.h"
#include "cxxabi_forced.h"

namespace std _GLIBCXX_VISIBILITY(default)
{
_GLIBCXX_BEGIN_NAMESPACE_VERSION

#if _GLIBCXX_USE_CXX11_ABI

  template<typename _CharT, typename _Traits, typename _Alloc>
    const typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::npos;

  template<typename _CharT, typename _Traits, typename _Alloc>
    void
    basic_string<_CharT, _Traits, _Alloc>::
    swap(basic_string& __s) _GLIBCXX_NOEXCEPT
    {
      if (this == &__s)
	return;

      _Alloc_traits::_S_on_swap;

      if (_M_is_local)
	if (__s._M_is_local)
	  
	    if (length && __s.length)
	      {
		_CharT __tmp_data[_S_local_capacity + 1];
		traits_type::copy;
		traits_type::copy;
		traits_type::copy;
	      }
	    else if (__s.length)
	      {
		traits_type::copy;
		_M_length;
		__s._M_set_length;
		return;
	      }
	    else if (length)
	      {
		traits_type::copy;
		__s._M_length;
		_M_set_length;
		return;
	      }
	  
	else
	  {
	    const size_type __tmp_capacity = __s._M_allocated_capacity;
	    traits_type::copy;
	    _M_data;
	    __s._M_data;
	    _M_capacity;
	  }
      else
	
	  const size_type __tmp_capacity = _M_allocated_capacity;
	  if (__s._M_is_local)
	    {
	      traits_type::copy;
	      __s._M_data;
	      _M_data;
	    }
	  else
	    
	      pointer __tmp_ptr = _M_data;
	      _M_data;
	      __s._M_data;
	      _M_capacity;
	    
	  __s._M_capacity;
	

      const size_type __tmp_length = length;
      _M_length;
      __s._M_length;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::pointer
    basic_string<_CharT, _Traits, _Alloc>::
    _M_create(size_type& __capacity, size_type __old_capacity)
    {
      // _GLIBCXX_RESOLVE_LIB_DEFECTS
      // 83.  String::npos vs. string::max_size
      if (__capacity > max_size)
	std::__throw_length_error;

      // The below implements an exponential growth policy, necessary to
      // meet amortized linear time requirements of the library: see
      // http://gcc.gnu.org/ml/libstdc++/2001-07/msg00085.html.
      if (__capacity > __old_capacity && __capacity < 2 * __old_capacity)
	
	  __capacity = 2 * __old_capacity;
	  // Never allocate a string bigger than max_size.
	  if (__capacity > max_size)
	    __capacity = max_size;
	

      // NB: Need an array of char_type[__capacity], plus a terminating
      // null char_type element.
      return _Alloc_traits::allocate;
    }

  // NB: This is the special case for Input Iterators, used in
  // istreambuf_iterators, etc.
  // Input Iterators have a cost structure very different from
  // pointers, calling for a different coding style.
  template<typename _CharT, typename _Traits, typename _Alloc>
    template<typename _InIterator>
      void
      basic_string<_CharT, _Traits, _Alloc>::
      _M_construct(_InIterator __beg, _InIterator __end,
		   std::input_iterator_tag)
      {
	size_type __len = 0;
	size_type __capacity = size_type(_S_local_capacity);

	while (__beg != __end && __len < __capacity)
	  

	__try
	  {
	    while (__beg != __end)
	      
		if (__len == __capacity)
		  
		    // Allocate more space.
		    __capacity = __len + 1;
		    pointer __another = _M_create;
		    this->_S_copy;
		    _M_dispose;
		    _M_data;
		    _M_capacity;
		  
		_M_data[__len++] = *__beg;
		++__beg;
	      
	  }
	__catch(...)
	  {
	    _M_dispose;
	    __throw_exception_again;
	  }

	_M_set_length;
      }

  template<typename _CharT, typename _Traits, typename _Alloc>
    template<typename _InIterator>
      void
      basic_string<_CharT, _Traits, _Alloc>::
      _M_construct(_InIterator __beg, _InIterator __end,
		   std::forward_iterator_tag)
      {
	// NB: Not required, but considered best practice.
	if (__gnu_cxx::__is_null_pointer(__beg) && __beg != __end)
	  std::__throw_logic_error;

	size_type __dnew = static_cast<size_type>(std::distance(__beg, __end));

	if (__dnew > size_type(_S_local_capacity))
	  

	// Check for out_of_range and length_error exceptions.
	__try
	  { this->_S_copy_chars(_M_data(), __beg, __end); }
	__catch(...)
	  {
	    _M_dispose();
	    __throw_exception_again;
	  }

	_M_set_length(__dnew);
      }

  template<typename _CharT, typename _Traits, typename _Alloc>
    void
    basic_string<_CharT, _Traits, _Alloc>::
    _M_construct(size_type __n, _CharT __c)
    {
      if (__n > size_type(_S_local_capacity))
	

      if (__n)
	this->_S_assign;

      _M_set_length;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    void
    basic_string<_CharT, _Traits, _Alloc>::
    _M_assign(const basic_string& __str)
    {
      if (this != &__str)
	{
	  const size_type __rsize = __str.length;
	  const size_type __capacity = capacity;

	  if (__rsize > __capacity)
	    
	      size_type __new_capacity = __rsize;
	      pointer __tmp = _M_create;
	      _M_dispose;
	      _M_data;
	      _M_capacity;
	    

	  if (__rsize)
	    this->_S_copy;

	  _M_set_length;
	}
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    void
    basic_string<_CharT, _Traits, _Alloc>::
    reserve(size_type __res)
    {
      // Make sure we don't shrink below the current size.
      if (__res < length)
	__res = length;

      const size_type __capacity = capacity;
      if (__res != __capacity)
	
	  if (__res > __capacity
	      || __res > size_type(_S_local_capacity))
	    {
	      pointer __tmp = _M_create;
	      this->_S_copy;
	      _M_dispose;
	      _M_data;
	      _M_capacity;
	    }
	  else if (!_M_is_local)
	    
	      this->_S_copy;
	      _M_destroy;
	      _M_data;
	    
	
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    void
    basic_string<_CharT, _Traits, _Alloc>::
    _M_mutate(size_type __pos, size_type __len1, const _CharT* __s,
	      size_type __len2)
    {
      const size_type __how_much = length - __pos - __len1;

      size_type __new_capacity = length + __len2 - __len1;
      pointer __r = _M_create;

      if (__pos)
	this->_S_copy;
      if (__s && __len2)
	this->_S_copy;
      if (__how_much)
	this->_S_copy;

      _M_dispose;
      _M_data;
      _M_capacity;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    void
    basic_string<_CharT, _Traits, _Alloc>::
    _M_erase(size_type __pos, size_type __n)
    {
      const size_type __how_much = length - __pos - __n;

      if (__how_much && __n)
	this->_S_move;

      _M_set_length;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    void
    basic_string<_CharT, _Traits, _Alloc>::
    resize(size_type __n, _CharT __c)
    {
      const size_type __size = this->size;
      if (__size < __n)
	this->append;
      else if (__n < __size)
	this->_M_erase;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    basic_string<_CharT, _Traits, _Alloc>&
    basic_string<_CharT, _Traits, _Alloc>::
    _M_append(const _CharT* __s, size_type __n)
    {
      const size_type __len = __n + this->size;

      if (__len = this-capacity)
	;
      else
	this->_M_mutate;

      this->_M_set_length;
      return *this;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    template<typename _InputIterator>
      basic_string<_CharT, _Traits, _Alloc>&
      basic_string<_CharT, _Traits, _Alloc>::
      _M_replace_dispatch(const_iterator __i1, const_iterator __i2,
			  _InputIterator __k1, _InputIterator __k2,
			  std::__false_type)
      {
	const basic_string __s;
	const size_type __n1 = __i2 - __i1;
	return _M_replace;
      }

  template<typename _CharT, typename _Traits, typename _Alloc>
    basic_string<_CharT, _Traits, _Alloc>&
    basic_string<_CharT, _Traits, _Alloc>::
    _M_replace_aux(size_type __pos1, size_type __n1, size_type __n2,
		   _CharT __c)
    {
      _M_check_length;

      const size_type __old_size = this->size;
      const size_type __new_size = __old_size + __n2 - __n1;

      if (__new_size = this-capacity)
	
	  pointer __p = this->_M_data + __pos1;

	  const size_type __how_much = __old_size - __pos1 - __n1;
	  if (__how_much && __n1 != __n2)
	    this->_S_move;
	
      else
	this->_M_mutate;

      if (__n2)
	this->_S_assign;

      this->_M_set_length;
      return *this;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    basic_string<_CharT, _Traits, _Alloc>&
    basic_string<_CharT, _Traits, _Alloc>::
    _M_replace(size_type __pos, size_type __len1, const _CharT* __s,
	       const size_type __len2)
    {
      _M_check_length;

      const size_type __old_size = this->size;
      const size_type __new_size = __old_size + __len2 - __len1;

      if (__new_size = this-capacity)
	{
	  pointer __p = this->_M_data + __pos;

	  const size_type __how_much = __old_size - __pos - __len1;
	  if (_M_disjunct)
	    
	      if (__how_much && __len1 != __len2)
		this->_S_move;
	      if (__len2)
		this->_S_copy;
	    
	  else
	    
	      // Work in-place.
	      if (__len2 && __len2 = __len1)
		this-_S_move;
	      if (__how_much && __len1 != __len2)
		this->_S_move;
	      if (__len2 > __len1)
		
		  if (__s + __len2 = __p + __len1)
		    this-_S_move;
		  else if (__s >= __p + __len1)
		    this->_S_copy;
		  else
		    
		      const size_type __nleft =  - __s;
		      this->_S_move;
		      this->_S_copy;
		    
		
	    
	}
      else
	this->_M_mutate;

      this->_M_set_length;
      return *this;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    copy(_CharT* __s, size_type __n, size_type __pos) const
    {
      _M_check;
      __n = _M_limit;
      __glibcxx_requires_string_len(__s, __n);
      if (__n)
	_S_copy;
      // 21.3.5.7 par 3: do not append null.  
      return __n;
    }

#else  // !_GLIBCXX_USE_CXX11_ABI

  template
    const typename basic_string::size_type
    basic_string::
    _Rep::_S_max_size =  / 4;

  template
    const _CharT
    basic_string::
    _Rep::_S_terminal = _CharT;

  template
    const typename basic_string::size_type
    basic_string::npos;

  // Linker sets _S_empty_rep_storage to all 0s 
  // at static init time .
  template
    typename basic_string::size_type
    basic_string::_Rep::_S_empty_rep_storage[
     /
      sizeof];

  // NB: This is the special case for Input Iterators, used in
  // istreambuf_iterators, etc.
  // Input Iterators have a cost structure very different from
  // pointers, calling for a different coding style.
  template
    template
      _CharT*
      basic_string::
      _S_construct
      
#if _GLIBCXX_FULLY_DYNAMIC_STRING == 0
	if 
	  return _S_empty_rep._M_refdata;
#endif
	// Avoid reallocation for common case.
	_CharT __buf[128];
	size_type __len = 0;
	while 
	  
	_Rep* __r = _Rep::_S_create;
	_M_copy;
	__try
	  
	    while 
	      
		if 
		  
		    // Allocate more space.
		    _Rep* __another = _Rep::_S_create;
		    _M_copy;
		    __r->_M_destroy;
		    __r = __another;
		  
		__r->_M_refdata[__len++] = *__beg;
		++__beg;
	      
	  
	__catch
	  
	__r->_M_set_length_and_sharable;
	return __r->_M_refdata;
      

  template
    template 
      _CharT*
      basic_string::
      _S_construct
      
#if _GLIBCXX_FULLY_DYNAMIC_STRING == 0
	if 
	  return _S_empty_rep._M_refdata;
#endif
	// NB: Not required, but considered best practice.
	if 
	  __throw_logic_error;

	const size_type __dnew = static_cast;
	// Check for out_of_range and length_error exceptions.
	_Rep* __r = _Rep::_S_create;
	__try
	  
	__catch
	  
	__r->_M_set_length_and_sharable;
	return __r->_M_refdata;
      

  template
    _CharT*
    basic_string::
    _S_construct
    
#if _GLIBCXX_FULLY_DYNAMIC_STRING == 0
      if 
	return _S_empty_rep._M_refdata;
#endif
      // Check for out_of_range and length_error exceptions.
      _Rep* __r = _Rep::_S_create;
      if 
	_M_assign;

      __r->_M_set_length_and_sharable;
      return __r->_M_refdata;
    

  template
    basic_string::
    basic_string
    : _M_dataplus__str._M_rep->_M_grab,
		  __str.get_allocator
    

  template
    basic_string::
    basic_string
    : _M_dataplus
    

  template
    basic_string::
    basic_string
    : _M_dataplus_S_construct__str._M_data
			       + __str._M_check,
			       __str._M_data + __str._M_limit
			       + __pos, _Alloc, _Alloc
    

  template
    basic_string::
    basic_string
    : _M_dataplus_S_construct__str._M_data
			       + __str._M_check,
			       __str._M_data + __str._M_limit
			       + __pos, __a, __a
    

  // TBD: DPG annotate
  template
    basic_string::
    basic_string
    : _M_dataplus
    

  // TBD: DPG annotate
  template
    basic_string::
    basic_string
    : _M_dataplus
    

  template
    basic_string::
    basic_string
    : _M_dataplus
    

  // TBD: DPG annotate
  template
    template
    basic_string::
    basic_string
    : _M_dataplus
    

#if __cplusplus >= 201103L
  template
    basic_string::
    basic_string
    : _M_dataplus
    
#endif

  template
    basic_string&
    basic_string::
    assign
    
      if 
	
	  // XXX MT
	  const allocator_type __a = this->get_allocator;
	  _CharT* __tmp = __str._M_rep->_M_grab;
	  _M_rep->_M_dispose;
	  _M_data;
	
      return *this;
    

  template
    basic_string&
    basic_string::
    assign
    
      __glibcxx_requires_string_len;
      _M_check_length;
      if 
	return _M_replace_safe;
      else
	
	  // Work in-place.
	  const size_type __pos = __s - _M_data;
	  if 
	    _M_copy;
	  else if 
	    _M_move;
	  _M_rep->_M_set_length_and_sharable;
	  return *this;
	
     

  template
    basic_string&
    basic_string::
    append
    
      if 
	
	  _M_check_length;	  
	  const size_type __len = __n + this->size;
	  if 
	    this->reserve;
	  _M_assign;
	  _M_rep->_M_set_length_and_sharable;
	
      return *this;
    

  template
    basic_string&
    basic_string::
    append
    
      __glibcxx_requires_string_len;
      if 
	
	  _M_check_length;
	  const size_type __len = __n + this->size;
	  if 
	    
	      if 
		this->reserve;
	      else
		
		  const size_type __off = __s - _M_data;
		  this->reserve;
		  __s = _M_data + __off;
		
	    
	  _M_copy;
	  _M_rep->_M_set_length_and_sharable;
	
      return *this;
    

  template
    basic_string&
    basic_string::
    append
    
      const size_type __size = __str.size;
      if 
	
	  const size_type __len = __size + this->size;
	  if 
	    this->reserve;
	  _M_copy;
	  _M_rep->_M_set_length_and_sharable;
	
      return *this;
        

  template
    basic_string&
    basic_string::
    append
    
      __str._M_check;
      __n = __str._M_limit;
      if 
	
	  const size_type __len = __n + this->size;
	  if 
	    this->reserve;
	  _M_copy;
	  _M_rep->_M_set_length_and_sharable;	  
	
      return *this;
    

   template
     basic_string&
     basic_string::
     insert
     
       __glibcxx_requires_string_len;
       _M_check;
       _M_check_length;
       if 
         return _M_replace_safe;
       else
         
           // Work in-place.
           const size_type __off = __s - _M_data;
           _M_mutate;
           __s = _M_data + __off;
           _CharT* __p = _M_data + __pos;
           if 
             _M_copy;
           else if 
             _M_copy;
           else
             
	       const size_type __nleft = __p - __s;
               _M_copy;
               _M_copy;
             
           return *this;
         
     

   template
     typename basic_string::iterator
     basic_string::
     erase
     
       _GLIBCXX_DEBUG_PEDASSERT;

       // NB: This isn't just an optimization , it's also a correctness
       // issue vs MT, see libstdc++/40518.
       const size_type __size = __last - __first;
       if 
	 
	   const size_type __pos = __first - _M_ibegin;
	   _M_mutate;
	   _M_rep->_M_set_leaked;
	   return iterator;
	 
       else
	 return __first;
     

   template
     basic_string&
     basic_string::
     replace
     
       __glibcxx_requires_string_len;
       _M_check;
       __n1 = _M_limit;
       _M_check_length;
       bool __left;
       if 
         return _M_replace_safe;
       else if 
	 
	   // Work in-place: non-overlapping case.
	   size_type __off = __s - _M_data;
	   __off;
	   _M_mutate;
	   _M_copy;
	   return *this;
	 
       else
	 
	   // Todo: overlapping case.
	   const basic_string __tmp;
	   return _M_replace_safe;
	 
     

  template
    void
    basic_string::_Rep::
    _M_destroy throw 
    
      const size_type __size = sizeof +
	                        * sizeof;
      _Raw_bytes_alloc.deallocate;
    

  template
    void
    basic_string::
    _M_leak_hard
    
#if _GLIBCXX_FULLY_DYNAMIC_STRING == 0
      if 
	return;
#endif
      if 
	_M_mutate;
      _M_rep->_M_set_leaked;
    

  template
    void
    basic_string::
    _M_mutate
    
      const size_type __old_size = this->size;
      const size_type __new_size = __old_size + __len2 - __len1;
      const size_type __how_much = __old_size - __pos - __len1;

      if 
	
	  // Must reallocate.
	  const allocator_type __a = get_allocator;
	  _Rep* __r = _Rep::_S_create;

	  if 
	    _M_copy;
	  if 
	    _M_copy;

	  _M_rep->_M_dispose;
	  _M_data;
	
      else if 
	
	  // Work in-place.
	  _M_move;
	
      _M_rep->_M_set_length_and_sharable;
    

  template
    void
    basic_string::
    reserve
    
      if 
        
	  // Make sure we don't shrink below the current size
	  if 
	    __res = this->size;
	  const allocator_type __a = get_allocator;
	  _CharT* __tmp = _M_rep->_M_clone;
	  _M_rep->_M_dispose;
	  _M_data;
        
    

  template
    void
    basic_string::
    swap
    
      if 
	_M_rep->_M_set_sharable;
      if 
	__s._M_rep->_M_set_sharable;
      if 
	
      // The code below can usually be optimized away.
      else
	
	  const basic_string __tmp1;
	  const basic_string __tmp2;
	  *this = __tmp2;
	  __s = __tmp1;
	
    

  template
    typename basic_string::_Rep*
    basic_string::_Rep::
    _S_create
    
      // _GLIBCXX_RESOLVE_LIB_DEFECTS
      // 83.  String::npos vs. string::max_size
      if 
	__throw_length_error;

      // The standard places no restriction on allocating more memory
      // than is strictly needed within this layer at the moment or as
      // requested by an explicit application call to reserve.

      // Many malloc implementations perform quite poorly when an
      // application attempts to allocate memory in a stepwise fashion
      // growing each allocation size by only 1 char.  Additionally,
      // it makes little sense to allocate less linear memory than the
      // natural blocking size of the malloc implementation.
      // Unfortunately, we would need a somewhat low-level calculation
      // with tuned parameters to get this perfect for any particular
      // malloc implementation.  Fortunately, generalizations about
      // common features seen among implementations seems to suffice.

      // __pagesize need not match the actual VM page size for good
      // results in practice, thus we pick a common value on the low
      // side.  __malloc_header_size is an estimate of the amount of
      // overhead per memory allocation .  According to folklore,
      // picking this value on the high side is better than
      // low-balling it especially when this algorithm is used with
      // malloc implementations that allocate memory blocks rounded up
      // to a size which is a power of 2.
      const size_type __pagesize = 4096;
      const size_type __malloc_header_size = 4 * sizeof;

      // The below implements an exponential growth policy, necessary to
      // meet amortized linear time requirements of the library: see
      // http://gcc.gnu.org/ml/libstdc++/2001-07/msg00085.html.
      // It's active for allocations requiring an amount of memory above
      // system pagesize. This is consistent with the requirements of the
      // standard: http://gcc.gnu.org/ml/libstdc++/2001-07/msg00130.html
      if 
	__capacity = 2 * __old_capacity;

      // NB: Need an array of char_type[__capacity], plus a terminating
      // null char_type element, plus enough for the _Rep data structure.
      // Whew. Seemingly so needy, yet so elemental.
      size_type __size =  * sizeof + sizeof;

      const size_type __adj_size = __size + __malloc_header_size;
      if 
	
	  const size_type __extra = __pagesize - __adj_size % __pagesize;
	  __capacity += __extra / sizeof;
	  // Never allocate a string bigger than _S_max_size.
	  if 
	    __capacity = _S_max_size;
	  __size =  * sizeof + sizeof;
	

      // NB: Might throw, but no worries about a leak, mate: _Rep
      // does not throw.
      void* __place = _Raw_bytes_alloc.allocate;
      _Rep *__p = new  _Rep;
      __p->_M_capacity = __capacity;
      // ABI compatibility - 3.4.x set in _S_create both
      // _M_refcount and _M_length.  All callers of _S_create
      // in basic_string.tcc then set just _M_length.
      // In 4.0.x and later both _M_refcount and _M_length
      // are initialized in the callers, unfortunately we can
      // have 3.4.x compiled code with _S_create callers inlined
      // calling 4.0.x+ _S_create.
      __p->_M_set_sharable;
      return __p;
    

  template
    _CharT*
    basic_string::_Rep::
    _M_clone
    
      // Requested capacity of the clone.
      const size_type __requested_cap = this->_M_length + __res;
      _Rep* __r = _Rep::_S_create;
      if 
	_M_copy;

      __r->_M_set_length_and_sharable;
      return __r->_M_refdata;
    

  template
    void
    basic_string::
    resize
    
      const size_type __size = this->size;
      _M_check_length;
      if 
	this->append;
      else if 
	this->erase;
      // else nothing 
    

  template
    template
      basic_string&
      basic_string::
      _M_replace_dispatch
      
	const basic_string __s;
	const size_type __n1 = __i2 - __i1;
	_M_check_length;
	return _M_replace_safe;
      

  template
    basic_string&
    basic_string::
    _M_replace_aux
    
      _M_check_length;
      _M_mutate;
      if 
	_M_assign;
      return *this;
    

  template
    basic_string&
    basic_string::
    _M_replace_safe
    
      _M_mutate;
      if 
	_M_copy;
      return *this;
    

    template
    typename basic_string::size_type
    basic_string::
    copy const
    
      _M_check;
      __n = _M_limit;
      __glibcxx_requires_string_len;
      if 
	_M_copy;
      // 21.3.5.7 par 3: do not append null.  
      return __n;
    
#endif  // !_GLIBCXX_USE_CXX11_ABI
   
  template<typename _CharT, typename _Traits, typename _Alloc>
    basic_string<_CharT, _Traits, _Alloc>
    operator+(const _CharT* __lhs,
	      const basic_string<_CharT, _Traits, _Alloc>& __rhs)
    {
      __glibcxx_requires_string(__lhs);
      typedef basic_string<_CharT, _Traits, _Alloc> __string_type;
      typedef typename __string_type::size_type	  __size_type;
      const __size_type __len = _Traits::length;
      __string_type __str;
      __str.reserve;
      __str.append;
      __str.append;
      return __str;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    basic_string<_CharT, _Traits, _Alloc>
    operator+(_CharT __lhs, const basic_string<_CharT, _Traits, _Alloc>& __rhs)
    {
      typedef basic_string<_CharT, _Traits, _Alloc> __string_type;
      typedef typename __string_type::size_type	  __size_type;
      __string_type __str;
      const __size_type __len = __rhs.size;
      __str.reserve;
      __str.append;
      __str.append;
      return __str;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    find(const _CharT* __s, size_type __pos, size_type __n) const
    {
      __glibcxx_requires_string_len(__s, __n);
      const size_type __size = this->size;
      const _CharT* __data = _M_data;

      if (__n == 0)
	return __pos <= npos;

      if (__n = __size)
	
	  for (; __pos = __size - __n; ++__pos)
	    if (traits_type::eq
		&& traits_type::compare == 0)
	      return __pos;
	
      return npos;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    find(_CharT __c, size_type __pos) const _GLIBCXX_NOEXCEPT
    {
      size_type __ret = npos;
      const size_type __size = this-size;
      if (__pos < __size)
	{
	  const _CharT* __data = _M_data;
	  const size_type __n = __size - __pos;
	  const _CharT* __p = traits_type::find;
	  if (__p)
	    __ret = __p - __data;
	}
      return __ret;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    rfind(const _CharT* __s, size_type __pos, size_type __n) const
    {
      __glibcxx_requires_string_len(__s, __n);
      const size_type __size = this->size;
      if (__n <= __size)
	
	  __pos = std::min;
	  const _CharT* __data = _M_data;
	  do
	    ;
	  while (__pos-- > 0);
	
      return npos;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    rfind(_CharT __c, size_type __pos) const _GLIBCXX_NOEXCEPT
    {
      size_type __size = this-size;
      if (__size)
	
	  if (--__size > __pos)
	    __size = __pos;
	  for (++__size; __size-- > 0; )
	    if (traits_type::eq)
	      return __size;
	
      return npos;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    find_first_of(const _CharT* __s, size_type __pos, size_type __n) const
    {
      __glibcxx_requires_string_len(__s, __n);
      for (; __n && __pos < this->size; ++__pos)
	{
	  const _CharT* __p = traits_type::find;
	  if (__p)
	    return __pos;
	}
      return npos;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    find_last_of(const _CharT* __s, size_type __pos, size_type __n) const
    {
      __glibcxx_requires_string_len(__s, __n);
      size_type __size = this->size;
      if (__size && __n)
	
	  if (--__size > __pos)
	    __size = __pos;
	  do
	    ;
	  while (__size-- != 0);
	
      return npos;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    find_first_not_of(const _CharT* __s, size_type __pos, size_type __n) const
    {
      __glibcxx_requires_string_len(__s, __n);
      for (; __pos < this->size; ++__pos)
	if (!traits_type::find)
	  return __pos;
      return npos;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    find_first_not_of(_CharT __c, size_type __pos) const _GLIBCXX_NOEXCEPT
    {
      for (; __pos < this->size; ++__pos)
	if (!traits_type::eq)
	  return __pos;
      return npos;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    find_last_not_of(const _CharT* __s, size_type __pos, size_type __n) const
    {
      __glibcxx_requires_string_len(__s, __n);
      size_type __size = this->size;
      if (__size)
	
	  if (--__size > __pos)
	    __size = __pos;
	  do
	    ;
	  while (__size--);
	
      return npos;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    typename basic_string<_CharT, _Traits, _Alloc>::size_type
    basic_string<_CharT, _Traits, _Alloc>::
    find_last_not_of(_CharT __c, size_type __pos) const _GLIBCXX_NOEXCEPT
    {
      size_type __size = this->size;
      if (__size)
	
	  if (--__size > __pos)
	    __size = __pos;
	  do
	    ;
	  while (__size--);
	
      return npos;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    int
    basic_string<_CharT, _Traits, _Alloc>::
    compare(size_type __pos, size_type __n, const basic_string& __str) const
    {
      _M_check;
      __n = _M_limit;
      const size_type __osize = __str.size;
      const size_type __len = std::min;
      int __r = traits_type::compare;
      if (!__r)
	__r = _S_compare;
      return __r;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    int
    basic_string<_CharT, _Traits, _Alloc>::
    compare(size_type __pos1, size_type __n1, const basic_string& __str,
	    size_type __pos2, size_type __n2) const
    {
      _M_check;
      __str._M_check;
      __n1 = _M_limit;
      __n2 = __str._M_limit;
      const size_type __len = std::min;
      int __r = traits_type::compare;
      if (!__r)
	__r = _S_compare;
      return __r;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    int
    basic_string<_CharT, _Traits, _Alloc>::
    compare(const _CharT* __s) const
    {
      __glibcxx_requires_string(__s);
      const size_type __size = this->size;
      const size_type __osize = traits_type::length;
      const size_type __len = std::min;
      int __r = traits_type::compare;
      if (!__r)
	__r = _S_compare;
      return __r;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    int
    basic_string <_CharT, _Traits, _Alloc>::
    compare(size_type __pos, size_type __n1, const _CharT* __s) const
    {
      __glibcxx_requires_string(__s);
      _M_check;
      __n1 = _M_limit;
      const size_type __osize = traits_type::length;
      const size_type __len = std::min;
      int __r = traits_type::compare;
      if (!__r)
	__r = _S_compare;
      return __r;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    int
    basic_string <_CharT, _Traits, _Alloc>::
    compare(size_type __pos, size_type __n1, const _CharT* __s,
	    size_type __n2) const
    {
      __glibcxx_requires_string_len(__s, __n2);
      _M_check;
      __n1 = _M_limit;
      const size_type __len = std::min;
      int __r = traits_type::compare;
      if (!__r)
	__r = _S_compare;
      return __r;
    }

  // 21.3.7.9 basic_string::getline and operators
  template<typename _CharT, typename _Traits, typename _Alloc>
    basic_istream<_CharT, _Traits>&
    operator>>(basic_istream<_CharT, _Traits>& __in,
	       basic_string<_CharT, _Traits, _Alloc>& __str)
    {
      typedef basic_istream<_CharT, _Traits>		__istream_type;
      typedef basic_string<_CharT, _Traits, _Alloc>	__string_type;
      typedef typename __istream_type::ios_base         __ios_base;
      typedef typename __istream_type::int_type		__int_type;
      typedef typename __string_type::size_type		__size_type;
      typedef ctype<_CharT>				__ctype_type;
      typedef typename __ctype_type::ctype_base         __ctype_base;

      __size_type __extracted = 0;
      typename __ios_base::iostate __err = __ios_base::goodbit;
      typename __istream_type::sentry __cerb;
      if (__cerb)
	
	  __try
	    {
	      // Avoid reallocation for common case.
	      __str.erase;
	      _CharT __buf[128];
	      __size_type __len = 0;	      
	      const streamsize __w = __in.width;
	      const __size_type __n = __w > 0 ? static_cast<__size_type>(__w)
		                              : __str.max_size;
	      const __ctype_type& __ct = use_facet;
	      const __int_type __eof = _Traits::eof;
	      __int_type __c = __in.rdbuf->sgetc;

	      while (__extracted < __n
		     && !_Traits::eq_int_type
		     && !__ct.is)
		
		  if (__len == sizeof(__buf) / sizeof(_CharT))
		    
		  __buf[__len++] = _Traits::to_char_type;
		  ++__extracted;
		  __c = __in.rdbuf->snextc;
		
	      __str.append;

	      if (_Traits::eq_int_type)
		__err |= __ios_base::eofbit;
	      __in.width;
	    }
	  __catch(__cxxabiv1::__forced_unwind&)
	    {
	      __in._M_setstate;
	      __throw_exception_again;
	    }
	  __catch(...)
	    {
	      // _GLIBCXX_RESOLVE_LIB_DEFECTS
	      // 91. Description of operator>> and getline for string
	      // might cause endless loop
	      __in._M_setstate;
	    }
	
      // 211.  operator>> doesn't set failbit
      if (!__extracted)
	__err |= __ios_base::failbit;
      if (__err)
	__in.setstate;
      return __in;
    }

  template<typename _CharT, typename _Traits, typename _Alloc>
    basic_istream<_CharT, _Traits>&
    getline(basic_istream<_CharT, _Traits>& __in,
	    basic_string<_CharT, _Traits, _Alloc>& __str, _CharT __delim)
    {
      typedef basic_istream<_CharT, _Traits>		__istream_type;
      typedef basic_string<_CharT, _Traits, _Alloc>	__string_type;
      typedef typename __istream_type::ios_base         __ios_base;
      typedef typename __istream_type::int_type		__int_type;
      typedef typename __string_type::size_type		__size_type;

      __size_type __extracted = 0;
      const __size_type __n = __str.max_size;
      typename __ios_base::iostate __err = __ios_base::goodbit;
      typename __istream_type::sentry __cerb;
      if (__cerb)
	
	  __try
	    {
	      __str.erase;
	      const __int_type __idelim = _Traits::to_int_type;
	      const __int_type __eof = _Traits::eof;
	      __int_type __c = __in.rdbuf->sgetc;

	      while (__extracted < __n
		     && !_Traits::eq_int_type
		     && !_Traits::eq_int_type)
		

	      if (_Traits::eq_int_type)
		__err |= __ios_base::eofbit;
	      else if (_Traits::eq_int_type)
		;
	      else
		__err |= __ios_base::failbit;
	    }
	  __catch(__cxxabiv1::__forced_unwind&)
	    {
	      __in._M_setstate;
	      __throw_exception_again;
	    }
	  __catch(...)
	    {
	      // _GLIBCXX_RESOLVE_LIB_DEFECTS
	      // 91. Description of operator>> and getline for string
	      // might cause endless loop
	      __in._M_setstate;
	    }
	
      if (!__extracted)
	__err |= __ios_base::failbit;
      if (__err)
	__in.setstate;
      return __in;
    }

  // Inhibit implicit instantiations for required instantiations,
  // which are defined via explicit instantiations elsewhere.
#if _GLIBCXX_EXTERN_TEMPLATE > 0
  extern template class basic_string<char>;
  extern template
    basic_istream<char>&
    operator>>(basic_istream<char>&, string&);
  extern template
    basic_ostream<char>&
    operator<<(basic_ostream<char>&, const string&);
  extern template
    basic_istream<char>&
    getline(basic_istream<char>&, string&, char);
  extern template
    basic_istream<char>&
    getline(basic_istream<char>&, string&);

#ifdef _GLIBCXX_USE_WCHAR_T
  extern template class basic_string<wchar_t>;
  extern template
    basic_istream<wchar_t>&
    operator>>(basic_istream<wchar_t>&, wstring&);
  extern template
    basic_ostream<wchar_t>&
    operator<<(basic_ostream<wchar_t>&, const wstring&);
  extern template
    basic_istream<wchar_t>&
    getline(basic_istream<wchar_t>&, wstring&, wchar_t);
  extern template
    basic_istream<wchar_t>&
    getline(basic_istream<wchar_t>&, wstring&);
#endif
#endif

_GLIBCXX_END_NAMESPACE_VERSION
} // namespace std

#endif
 