 /* Copyright  1992-2014 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or  any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   .  */

#ifndef	_SYS_CDEFS_H
#define	_SYS_CDEFS_H	1

/* We are almost always included from features.h. */
#ifndef _FEATURES_H
#pragma "/usr/include/features.h"
#include "features.h"
#endif

/* The GNU libc does not support any K&R compilers or the traditional mode
   of ISO C compilers anymore.  Check for some of the combinations not
   anymore supported.  */
#if defined __GNUC__ && !defined __STDC__
# error "You need a ISO C conforming compiler to use the glibc headers"
#endif

/* Some user header file might have defined this before.  */
#undef	__P
#undef	__PMT

#ifdef __GNUC__

/* All functions, except those with callbacks or those that
   synchronize memory, are leaf functions.  */
# if __GNUC_PREREQ  && !defined _LIBC
#  define __LEAF , __leaf__
#  define __LEAF_ATTR __attribute__ 
# else
#  define __LEAF
#  define __LEAF_ATTR
# endif

/* GCC can always grok prototypes.  For C++ programs we add throw
   to help it optimize the function calls.  But this works only with
   gcc 2.8.x and egcs.  For gcc 3.2 and up we even mark C functions
   as non-throwing using a function attribute since programs can use
   the -fexceptions options for C code as well.  */
# if !defined __cplusplus && __GNUC_PREREQ 
#  define __THROW	__attribute__ 
#  define __THROWNL	__attribute__ 
#  define __NTH	__attribute__  fct
# else
#  if defined __cplusplus && __GNUC_PREREQ 
#   define __THROW	throw 
#   define __THROWNL	throw 
#   define __NTH	__LEAF_ATTR fct throw 
#  else
#   define __THROW
#   define __THROWNL
#   define __NTHfct	fct
#  endif
# endif

#else	/* Not GCC.  */

# define __inline		/* No inline functions.  */

# define __THROW
# define __THROWNL
# define __NTH	fct

#endif	/* GCC.  */

/* These two macros are not used in glibc anymore.  They are kept here
   only because some other projects expect the macros to be defined.  */
#define __P	args
#define __PMT	args

/* For these things, GCC behaves the ANSI way normally,
   and the non-ANSI way under -traditional.  */

#define __CONCAT	x ## y
#define __STRING	#x

/* This is not a typedef so `const __ptr_t' does the right thing.  */
#define __ptr_t void *
#define __long_double_t  long double


/* C++ needs to know that types and declarations are C, not C++.  */
#ifdef	__cplusplus
# define __BEGIN_DECLS	extern "C" {
# define __END_DECLS	}
#else
# define __BEGIN_DECLS
# define __END_DECLS
#endif


/* The standard library needs the functions from the ISO C90 standard
   in the std namespace.  At the same time we want to be safe for
   future changes and we include the ISO C99 code in the non-standard
   namespace __c99.  The C++ wrapper header take case of adding the
   definitions to the global namespace.  */
#if defined __cplusplus && defined _GLIBCPP_USE_NAMESPACES
# define __BEGIN_NAMESPACE_STD	namespace std 
# define __USING_NAMESPACE_STD using std::name;
# define __BEGIN_NAMESPACE_C99	namespace __c99 
# define __USING_NAMESPACE_C99 using __c99::name;
#else
/* For compatibility we do not add the declarations into any
   namespace.  They will end up in the global namespace which is what
   old code expects.  */
# define __BEGIN_NAMESPACE_STD
# define __END_NAMESPACE_STD
# define __USING_NAMESPACE_STD(name)
# define __BEGIN_NAMESPACE_C99
# define __END_NAMESPACE_C99
# define __USING_NAMESPACE_C99(name)
#endif


/* Fortify support.  */
#define __bos __builtin_object_size 
#define __bos0 __builtin_object_size 
#define __fortify_function __extern_always_inline __attribute_artificial__

#if __GNUC_PREREQ 
# define __warndecl \
  extern void name  __attribute__
# define __warnattr __attribute__
# define __errordecl \
  extern void name  __attribute__
#else
# define __warndecl extern void name 
# define __warnattr
# define __errordecl extern void name 
#endif

/* Support for flexible arrays.  */
#if __GNUC_PREREQ 
/* GCC 2.97 supports C99 flexible array members.  */
# define __flexarr	[]
#else
# ifdef __GNUC__
#  define __flexarr	[0]
# else
#  if defined __STDC_VERSION__ && __STDC_VERSION__ >= 199901L
#   define __flexarr	[]
#  else
/* Some other non-C99 compiler.  Approximate with [1].  */
#   define __flexarr	[1]
#  endif
# endif
#endif


/* __asm__  is used throughout the headers to rename functions
   at the assembly language level.  This is wrapped by the __REDIRECT
   macro, in order to support compilers that can do this some other
   way.  When compilers don't support asm-names at all, we have to do
   preprocessor tricks instead .

   Example:
   int __REDIRECT; */

#if defined __GNUC__ && __GNUC__ >= 2

# define __REDIRECT name proto __asm__ 
# ifdef __cplusplus
#  define __REDIRECT_NTH \
     name proto __THROW __asm__ 
#  define __REDIRECT_NTHNL \
     name proto __THROWNL __asm__ 
# else
#  define __REDIRECT_NTH \
     name proto __asm__  __THROW
#  define __REDIRECT_NTHNL \
     name proto __asm__  __THROWNL
# endif
# define __ASMNAME  __ASMNAME2 
# define __ASMNAME2 __STRING  cname

/*
#elif __SOME_OTHER_COMPILER__

# define __REDIRECT name proto; \
	_Pragma
*/
#endif

/* GCC has various useful declarations that can be made with the
   `__attribute__' syntax.  All of the ways we use this do fine if
   they are omitted for compilers that don't understand it. */
#if !defined __GNUC__ || __GNUC__ < 2
# define __attribute__	/* Ignore */
#endif

/* At some point during the gcc 2.96 development the `malloc' attribute
   for functions was introduced.  We don't want to use it unconditionally
    since it generates warnings.  */
#if __GNUC_PREREQ 
# define __attribute_malloc__ __attribute__ 
#else
# define __attribute_malloc__ /* Ignore */
#endif

/* Tell the compiler which arguments to an allocation function
   indicate the size of the allocation.  */
#if __GNUC_PREREQ 
# define __attribute_alloc_size__ \
  __attribute__ 
#else
# define __attribute_alloc_size__(params) /* Ignore.  */
#endif

/* At some point during the gcc 2.96 development the `pure' attribute
   for functions was introduced.  We don't want to use it unconditionally
    since it generates warnings.  */
#if __GNUC_PREREQ 
# define __attribute_pure__ __attribute__ 
#else
# define __attribute_pure__ /* Ignore */
#endif

/* This declaration tells the compiler that the value is constant.  */
#if __GNUC_PREREQ 
# define __attribute_const__ __attribute__ 
#else
# define __attribute_const__ /* Ignore */
#endif

/* At some point during the gcc 3.1 development the `used' attribute
   for functions was introduced.  We don't want to use it unconditionally
    since it generates warnings.  */
#if __GNUC_PREREQ 
# define __attribute_used__ __attribute__ 
# define __attribute_noinline__ __attribute__ 
#else
# define __attribute_used__ __attribute__ 
# define __attribute_noinline__ /* Ignore */
#endif

/* gcc allows marking deprecated functions.  */
#if __GNUC_PREREQ 
# define __attribute_deprecated__ __attribute__ 
#else
# define __attribute_deprecated__ /* Ignore */
#endif

/* At some point during the gcc 2.8 development the `format_arg' attribute
   for functions was introduced.  We don't want to use it unconditionally
    since it generates warnings.
   If several `format_arg' attributes are given for the same function, in
   gcc-3.0 and older, all but the last one are ignored.  In newer gccs,
   all designated arguments are considered.  */
#if __GNUC_PREREQ 
# define __attribute_format_arg__ __attribute__ 
#else
# define __attribute_format_arg__ /* Ignore */
#endif

/* At some point during the gcc 2.97 development the `strfmon' format
   attribute for functions was introduced.  We don't want to use it
   unconditionally  since it
   generates warnings.  */
#if __GNUC_PREREQ 
# define __attribute_format_strfmon__ \
  __attribute__ 
#else
# define __attribute_format_strfmon__ /* Ignore */
#endif

/* The nonull function attribute allows to mark pointer parameters which
   must not be NULL.  */
#if __GNUC_PREREQ 
# define __nonnull __attribute__ 
#else
# define __nonnull(params)
#endif

/* If fortification mode, we warn about unused results of certain
   function calls which can lead to problems.  */
#if __GNUC_PREREQ 
# define __attribute_warn_unused_result__ \
   __attribute__ 
# if __USE_FORTIFY_LEVEL > 0
#  define __wur __attribute_warn_unused_result__
# endif
#else
# define __attribute_warn_unused_result__ /* empty */
#endif
#ifndef __wur
# define __wur /* Ignore */
#endif

/* Forces a function to be always inlined.  */
#if __GNUC_PREREQ 
# define __always_inline __inline __attribute__ 
#else
# define __always_inline __inline
#endif

/* Associate error messages with the source location of the call site rather
   than with the source location inside the function.  */
#if __GNUC_PREREQ 
# define __attribute_artificial__ __attribute__ 
#else
# define __attribute_artificial__ /* Ignore */
#endif

#ifdef __GNUC__
/* One of these will be defined if the __gnu_inline__ attribute is
   available.  In C++, __GNUC_GNU_INLINE__ will be defined even though
   __inline does not use the GNU inlining rules.  If neither macro is
   defined, this version of GCC only supports GNU inline semantics. */
# if defined __GNUC_STDC_INLINE__ || defined __GNUC_GNU_INLINE__
#  define __extern_inline extern __inline __attribute__ __gnu_inline__
#  define __extern_always_inline \
  extern __always_inline __attribute__ 
# else
#  define __extern_inline extern __inline
#  define __extern_always_inline extern __always_inline
# endif
#else /* Not GCC.  */
# define __extern_inline  /* Ignore */
# define __extern_always_inline /* Ignore */
#endif

/* GCC 4.3 and above allow passing all anonymous arguments of an
   __extern_always_inline function to some other vararg function.  */
#if __GNUC_PREREQ 
# define __va_arg_pack __builtin_va_arg_pack 
# define __va_arg_pack_len __builtin_va_arg_pack_len 
#endif

/* It is possible to compile containing GCC extensions even if GCC is
   run in pedantic mode if the uses are carefully marked using the
   `__extension__' keyword.  But this is not generally available before
   version 2.8.  */
#if !__GNUC_PREREQ 
# define __extension__		/* Ignore */
#endif

/* __restrict is known in EGCS 1.2 and above. */
#if !__GNUC_PREREQ 
# define __restrict	/* Ignore */
#endif

/* ISO C99 also allows to declare arrays as non-overlapping.  The syntax is
     array_name[restrict]
   GCC 3.1 supports this.  */
#if __GNUC_PREREQ  && !defined __GNUG__
# define __restrict_arr	__restrict
#else
# ifdef __GNUC__
#  define __restrict_arr	/* Not supported in old GCC.  */
# else
#  if defined __STDC_VERSION__ && __STDC_VERSION__ >= 199901L
#   define __restrict_arr	restrict
#  else
/* Some other non-C99 compiler.  */
#   define __restrict_arr	/* Not supported.  */
#  endif
# endif
#endif

#if __GNUC__ >= 3
# define __glibc_unlikely	__builtin_expect 
# define __glibc_likely	__builtin_expect 
#else
# define __glibc_unlikely	
# define __glibc_likely	
#endif

#pragma "/usr/include/x86_64-linux-gnu/bits/wordsize.h"
#include "wordsize.h"

#if defined __LONG_DOUBLE_MATH_OPTIONAL && defined __NO_LONG_DOUBLE_MATH
# define __LDBL_COMPAT 1
# ifdef __REDIRECT
#  define __LDBL_REDIR1 __REDIRECT 
#  define __LDBL_REDIR \
  __LDBL_REDIR1 
#  define __LDBL_REDIR1_NTH __REDIRECT_NTH 
#  define __LDBL_REDIR_NTH \
  __LDBL_REDIR1_NTH 
#  define __LDBL_REDIR1_DECL \
  extern __typeof  name __asm ;
#  define __LDBL_REDIR_DECL \
  extern __typeof  name __asm ;
#  define __REDIRECT_LDBL \
  __LDBL_REDIR1 
#  define __REDIRECT_NTH_LDBL \
  __LDBL_REDIR1_NTH 
# endif
#endif
#if !defined __LDBL_COMPAT || !defined __REDIRECT
# define __LDBL_REDIR1 name proto
# define __LDBL_REDIR name proto
# define __LDBL_REDIR1_NTH name proto __THROW
# define __LDBL_REDIR_NTH name proto __THROW
# define __LDBL_REDIR_DECL
# ifdef __REDIRECT
#  define __REDIRECT_LDBL __REDIRECT 
#  define __REDIRECT_NTH_LDBL \
  __REDIRECT_NTH 
# endif
#endif

#endif	 /* sys/cdefs.h */
 