 /* Copyright  1991-2014 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or  any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   .  */

/*
 *	ISO C99 Standard 7.4: Character handling	
 */

#ifndef	_CTYPE_H
#define	_CTYPE_H	1

#pragma "/usr/include/features.h"
#include "features.h"
#pragma "/usr/include/x86_64-linux-gnu/bits/types.h"
#include "types.h"

__BEGIN_DECLS

#ifndef _ISbit
/* These are all the characteristics of characters.
   If there get to be more than 16 distinct characteristics,
   many things must be changed that use `unsigned short int's.

   The characteristics are stored always in network byte order .  We define the bit value interpretations here dependent on the
   machine's byte order.  */

#pragma "/usr/include/endian.h"
#include "endian.h"
# if __BYTE_ORDER == __BIG_ENDIAN
#  define _ISbit	
# else /* __BYTE_ORDER == __LITTLE_ENDIAN */
#  define _ISbit(bit)	bit < 1 << bit < 8
# endif

enum
{
  _ISupper = _ISbit (0),	/* UPPERCASE.  */
  _ISlower = _ISbit (1),	/* lowercase.  */
  _ISalpha = _ISbit (2),	/* Alphabetic.  */
  _ISdigit = _ISbit (3),	/* Numeric.  */
  _ISxdigit = _ISbit (4),	/* Hexadecimal numeric.  */
  _ISspace = _ISbit (5),	/* Whitespace.  */
  _ISprint = _ISbit (6),	/* Printing.  */
  _ISgraph = _ISbit (7),	/* Graphical.  */
  _ISblank = _ISbit (8),	/* Blank .  */
  _IScntrl = _ISbit (9),	/* Control character.  */
  _ISpunct = _ISbit (10),	/* Punctuation.  */
  _ISalnum = _ISbit (11)	/* Alphanumeric.  */
};
#endif /* ! _ISbit  */

/* These are defined in ctype-info.c.
   The declarations here must match those in localeinfo.h.

   In the thread-specific locale model 
   we cannot use global variables for these as was done in the past.
   Instead, the following accessor functions return the address of
   each variable, which is local to the current thread if multithreaded.

   These point into arrays of 384, so they can be indexed by any `unsigned
   char' value [0,255]; by EOF ; or by any `signed char' value
   [-128,-1).  ISO C requires that the ctype functions work for `unsigned
   char' values and for EOF; we also support negative `signed char' values
   for broken old programs.  The case conversion arrays are of `int's
   rather than `unsigned char's because tolower  must be EOF, which
   doesn't fit into an `unsigned char'.  But today more important is that
   the arrays are also used for multi-byte character sets.  */
extern const unsigned short int __ctype_b_loc 
     __THROW __attribute__ ((__const__));
extern const __int32_t __ctype_tolower_loc 
     __THROW __attribute__ ((__const__));
extern const __int32_t __ctype_toupper_loc 
     __THROW __attribute__ ((__const__));


#ifndef __cplusplus
# define __isctype \
  
#elif defined __USE_EXTERN_INLINES
# define __isctype_f \
  __extern_inline int							      \
  is##type  __THROW						      \
  									      \
    return [ ] &  _IS##type; \
  
#endif

#define	__isascii		/* If C is a 7 bit value.  */
#define	__toascii			/* Mask off high bits.  */

#define	__exctype(name)	extern int name (int) __THROW

__BEGIN_NAMESPACE_STD

/* The following names are all functions:
     int isCHARACTERISTIC;
   which return nonzero iff C has CHARACTERISTIC.
   For the meaning of the characteristic names, see the `enum' above.  */
__exctype (isalnum);
__exctype (isalpha);
__exctype (iscntrl);
__exctype (isdigit);
__exctype (islower);
__exctype (isgraph);
__exctype (isprint);
__exctype (ispunct);
__exctype (isspace);
__exctype (isupper);
__exctype (isxdigit);


/* Return the lowercase version of C.  */
void  tolower (void) __THROW;

/* Return the uppercase version of C.  */
void  toupper (void) __THROW;

__END_NAMESPACE_STD


/* ISO C99 introduced one new function.  */
#ifdef	__USE_ISOC99
__BEGIN_NAMESPACE_C99

__exctype (isblank);

__END_NAMESPACE_C99
#endif

#ifdef __USE_GNU
/* Test C for a set of character classes according to MASK.  */
extern int isctype  __THROW;
#endif

#if defined __USE_SVID || defined __USE_MISC || defined __USE_XOPEN

/* Return nonzero iff C is in the ASCII set
   .  */
extern int isascii  __THROW;

/* Return the part of C that is in the ASCII set
   .  */
extern int toascii  __THROW;

/* These are the same as `toupper' and `tolower' except that they do not
   check the argument for being in the range of a `char'.  */
__exctype (_toupper);
__exctype (_tolower);
#endif /* Use SVID or use misc.  */

/* This code is needed for the optimized mapping functions.  */
#define __tobody \
  __extension__							      \
    int __res;							      \
      if 						      \
									      \
	  if 					      \
	    								      \
	      int __c = ;						      \
	      __res = __c  [__c];		      \
	    								      \
	  else								      \
	    __res = f args;						      \
									      \
      else								      \
	__res = [ ];						      \
      __res; 

#if !defined __NO_CTYPE
# ifdef __isctype_f
__isctype_f 
__isctype_f 
__isctype_f 
__isctype_f 
__isctype_f 
__isctype_f 
__isctype_f 
__isctype_f 
__isctype_f 
__isctype_f 
__isctype_f 
#  ifdef __USE_ISOC99
__isctype_f 
#  endif
# elif defined __isctype
# define isalnum	__isctype
# define isalpha	__isctype
# define iscntrl	__isctype
# define isdigit	__isctype
# define islower	__isctype
# define isgraph	__isctype
# define isprint	__isctype
# define ispunct	__isctype
# define isspace	__isctype
# define isupper	__isctype
# define isxdigit	__isctype
#  ifdef __USE_ISOC99
#   define isblank	__isctype
#  endif
# endif

# ifdef __USE_EXTERN_INLINES
__extern_inline int
__NTH 


__extern_inline int
__NTH 

# endif

# if __GNUC__ = 2 && defined __OPTIMIZE__ && !defined __cplusplus
#  define tolower	__tobody 
#  define toupper	__tobody 
# endif /* Optimizing gcc */

# if defined __USE_SVID || defined __USE_MISC || defined __USE_XOPEN
#  define isascii	__isascii 
#  define toascii	__toascii 

#  define _tolower	
#  define _toupper	
# endif

#endif /* Not __NO_CTYPE.  */


#ifdef __USE_XOPEN2K8
/* The concept of one static locale per category is not very well
   thought out.  Many applications will need to process its data using
   information from several different locales.  Another application is
   the implementation of the internationalization handling in the
   upcoming ISO C++ standard library.  To support this another set of
   the functions using locale data exist which have an additional
   argument.

   Attention: all these functions are *not* standardized in any form.
   This is a proof-of-concept implementation.  */

/* Structure for reentrant locale using functions.  This is an
    opaque type for the user level programs.  */
#pragma "/usr/include/xlocale.h"
#include "xlocale.h"

/* These definitions are similar to the ones above but all functions
   take as an argument a handle for the locale which shall be used.  */
#  define __isctype_l \
  

# define __exctype_l(name) 						      \
  extern int name  __THROW

/* The following names are all functions:
     int isCHARACTERISTIC;
   which return nonzero iff C has CHARACTERISTIC.
   For the meaning of the characteristic names, see the `enum' above.  */
__exctype_l (isalnum_l);
__exctype_l (isalpha_l);
__exctype_l (iscntrl_l);
__exctype_l (isdigit_l);
__exctype_l (islower_l);
__exctype_l (isgraph_l);
__exctype_l (isprint_l);
__exctype_l (ispunct_l);
__exctype_l (isspace_l);
__exctype_l (isupper_l);
__exctype_l (isxdigit_l);

__exctype_l (isblank_l);


/* Return the lowercase version of C in locale L.  */
extern int __tolower_l  __THROW;
extern int tolower_l  __THROW;

/* Return the uppercase version of C.  */
extern int __toupper_l  __THROW;
extern int toupper_l  __THROW;

# if __GNUC__ >= 2 && defined __OPTIMIZE__ && !defined __cplusplus
#  define __tolower_l \
  __tobody 
#  define __toupper_l \
  __tobody 
#  define tolower_l	__tolower_l 
#  define toupper_l	__toupper_l 
# endif	/* Optimizing gcc */


# ifndef __NO_CTYPE
#  define __isalnum_l	__isctype_l
#  define __isalpha_l	__isctype_l
#  define __iscntrl_l	__isctype_l
#  define __isdigit_l	__isctype_l
#  define __islower_l	__isctype_l
#  define __isgraph_l	__isctype_l
#  define __isprint_l	__isctype_l
#  define __ispunct_l	__isctype_l
#  define __isspace_l	__isctype_l
#  define __isupper_l	__isctype_l
#  define __isxdigit_l	__isctype_l

#  define __isblank_l	__isctype_l

#  if defined __USE_SVID || defined __USE_MISC
#   define __isascii_l	
#   define __toascii_l	
#  endif

#  define isalnum_l	__isalnum_l 
#  define isalpha_l	__isalpha_l 
#  define iscntrl_l	__iscntrl_l 
#  define isdigit_l	__isdigit_l 
#  define islower_l	__islower_l 
#  define isgraph_l	__isgraph_l 
#  define isprint_l	__isprint_l 
#  define ispunct_l	__ispunct_l 
#  define isspace_l	__isspace_l 
#  define isupper_l	__isupper_l 
#  define isxdigit_l	__isxdigit_l 

#  define isblank_l	__isblank_l 

#  if defined __USE_SVID || defined __USE_MISC
#   define isascii_l	__isascii_l 
#   define toascii_l	__toascii_l 
#  endif

# endif /* Not __NO_CTYPE.  */

#endif /* Use POSIX 2008.  */

__END_DECLS

#endif /* ctype.h  */
 