 // Debugging support implementation -*- C++ -*-

// Copyright  2003-2015 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or 
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Under Section 7 of GPL version 3, you are granted additional
// permissions described in the GCC Runtime Library Exception, version
// 3.1, as published by the Free Software Foundation.

// You should have received a copy of the GNU General Public License and
// a copy of the GCC Runtime Library Exception along with this program;
// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
// .

/** @file debug/debug.h
 *  This file is a GNU debug extension to the Standard C++ Library.
 */

#ifndef _GLIBCXX_DEBUG_MACRO_SWITCH_H
#define _GLIBCXX_DEBUG_MACRO_SWITCH_H 1

/** Macros and namespaces used by the implementation outside of debug
 *  wrappers to verify certain properties. The __glibcxx_requires_xxx
 *  macros are merely wrappers around the __glibcxx_check_xxx wrappers
 *  when we are compiling with debug mode, but disappear when we are
 *  in release mode so that there is no checking performed in, e.g.,
 *  the standard library algorithms.
*/

#pragma "/home/regehr/z/compiler-install/gcc-r231221-install/include/c++/6.0.0/debug/assertions.h"
#include "assertions.h"

// Debug mode namespaces.

/**
 * @namespace std::__debug
 * @brief GNU debug code, replaces standard behavior with debug behavior.
 */
namespace std
{ 
  namespace __debug { } 
}

/** @namespace __gnu_debug
 *  @brief GNU debug classes for public use.
*/
namespace __gnu_debug
{
  using namespace std::__debug;
}

#ifndef _GLIBCXX_DEBUG

# define __glibcxx_requires_cond(_Cond,_Msg)
# define __glibcxx_requires_valid_range
# define __glibcxx_requires_sorted
# define __glibcxx_requires_sorted_pred
# define __glibcxx_requires_sorted_set
# define __glibcxx_requires_sorted_set_pred
# define __glibcxx_requires_partitioned_lower
# define __glibcxx_requires_partitioned_upper
# define __glibcxx_requires_partitioned_lower_pred
# define __glibcxx_requires_partitioned_upper_pred
# define __glibcxx_requires_heap
# define __glibcxx_requires_heap_pred
# define __glibcxx_requires_string
# define __glibcxx_requires_string_len
# define __glibcxx_requires_subscript
# define __glibcxx_requires_irreflexive
# define __glibcxx_requires_irreflexive2
# define __glibcxx_requires_irreflexive_pred
# define __glibcxx_requires_irreflexive_pred2

#ifdef _GLIBCXX_ASSERTIONS
// Verify that [_First, _Last) forms a non-empty iterator range.
# define __glibcxx_requires_non_empty_range \
  __glibcxx_assert
// Verify that the container is nonempty
# define __glibcxx_requires_nonempty \
  __glibcxx_assert
#else
# define __glibcxx_requires_non_empty_range
# define __glibcxx_requires_nonempty
#endif

#else

#error localize_headers should not have tried to include debug/macros.h

# define __glibcxx_requires_cond _GLIBCXX_DEBUG_VERIFY
# define __glibcxx_requires_valid_range	\
  __glibcxx_check_valid_range
# define __glibcxx_requires_non_empty_range	\
  __glibcxx_check_non_empty_range
# define __glibcxx_requires_sorted	\
  __glibcxx_check_sorted
# define __glibcxx_requires_sorted_pred	\
  __glibcxx_check_sorted_pred
# define __glibcxx_requires_sorted_set	\
  __glibcxx_check_sorted_set
# define __glibcxx_requires_sorted_set_pred \
  __glibcxx_check_sorted_set_pred
# define __glibcxx_requires_partitioned_lower	\
  __glibcxx_check_partitioned_lower
# define __glibcxx_requires_partitioned_upper	\
  __glibcxx_check_partitioned_upper
# define __glibcxx_requires_partitioned_lower_pred \
  __glibcxx_check_partitioned_lower_pred
# define __glibcxx_requires_partitioned_upper_pred \
  __glibcxx_check_partitioned_upper_pred
# define __glibcxx_requires_heap	\
  __glibcxx_check_heap
# define __glibcxx_requires_heap_pred	\
  __glibcxx_check_heap_pred
# define __glibcxx_requires_nonempty __glibcxx_check_nonempty
# define __glibcxx_requires_string __glibcxx_check_string
# define __glibcxx_requires_string_len	\
  __glibcxx_check_string_len
# define __glibcxx_requires_subscript __glibcxx_check_subscript
# define __glibcxx_requires_irreflexive	\
  __glibcxx_check_irreflexive
# define __glibcxx_requires_irreflexive2	\
  __glibcxx_check_irreflexive2
# define __glibcxx_requires_irreflexive_pred	\
  __glibcxx_check_irreflexive_pred
# define __glibcxx_requires_irreflexive_pred2	\
  __glibcxx_check_irreflexive_pred2

#error localize_headers should not have tried to include debug/functions.h

#endif

#endif // _GLIBCXX_DEBUG_MACRO_SWITCH_H
 