 /* Copyright  2002-2014 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or  any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   .  */

#ifndef _PTHREAD_H
#define _PTHREAD_H	1

#pragma "/usr/include/features.h"
#include "features.h"
#pragma "/usr/include/endian.h"
#include "endian.h"
#pragma "/usr/include/sched.h"
#include "sched.h"
#pragma "/usr/include/time.h"
#include "time.h"

#pragma "/usr/include/x86_64-linux-gnu/bits/pthreadtypes.h"
#include "pthreadtypes.h"
#pragma "/usr/include/x86_64-linux-gnu/bits/setjmp.h"
#include "setjmp.h"
#pragma "/usr/include/x86_64-linux-gnu/bits/wordsize.h"
#include "wordsize.h"


/* Detach state.  */
enum
{
   
#define PTHREAD_CREATE_DETACHED	PTHREAD_CREATE_DETACHED
};


/* Mutex types.  */
enum
{
  PTHREAD_MUTEX_TIMED_NP,
  PTHREAD_MUTEX_RECURSIVE_NP,
  PTHREAD_MUTEX_ERRORCHECK_NP,
  PTHREAD_MUTEX_ADAPTIVE_NP
#if defined __USE_UNIX98 || defined __USE_XOPEN2K8
  ,
  PTHREAD_MUTEX_NORMAL = PTHREAD_MUTEX_TIMED_NP,
    PTHREAD_MUTEX_DEFAULT = PTHREAD_MUTEX_NORMAL
#endif
#ifdef __USE_GNU
  /* For compatibility.  */
  , PTHREAD_MUTEX_FAST_NP = PTHREAD_MUTEX_TIMED_NP
#endif
};


#ifdef __USE_XOPEN2K
/* Robust mutex or not flags.  */
enum
{
   
};
#endif


#if defined __USE_POSIX199506 || defined __USE_UNIX98
/* Mutex protocols.  */
enum
{
    
};
#endif


/* Mutex initializers.  */
#if __PTHREAD_MUTEX_HAVE_ELISION == 1 /* 64bit layout.  */
#define __PTHREAD_SPINS 0, 0
#elif __PTHREAD_MUTEX_HAVE_ELISION == 2 /* 32bit layout.  */
#define __PTHREAD_SPINS 
#else
#define __PTHREAD_SPINS 0
#endif

#ifdef __PTHREAD_MUTEX_HAVE_PREV
# define PTHREAD_MUTEX_INITIALIZER \
  
# ifdef __USE_GNU
#  define PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP \
  
#  define PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP \
  
#  define PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP \
  
#  define PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP \
  

# endif
#else
# define PTHREAD_MUTEX_INITIALIZER \
  
# ifdef __USE_GNU
#  define PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP \
  
#  define PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP \
  
#  define PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP \
  

# endif
#endif


/* Read-write lock types.  */
#if defined __USE_UNIX98 || defined __USE_XOPEN2K
enum
{
  
};

/* Define __PTHREAD_RWLOCK_INT_FLAGS_SHARED to 1 if pthread_rwlock_t
   has the shared field.  All 64-bit architectures have the shared field
   in pthread_rwlock_t.  */
#ifndef __PTHREAD_RWLOCK_INT_FLAGS_SHARED
# if __WORDSIZE == 64
#  define __PTHREAD_RWLOCK_INT_FLAGS_SHARED 1
# endif
#endif

/* Read-write lock initializers.  */
# define PTHREAD_RWLOCK_INITIALIZER \
  
# ifdef __USE_GNU
#  ifdef __PTHREAD_RWLOCK_INT_FLAGS_SHARED
#   define PTHREAD_RWLOCK_WRITER_NONRECURSIVE_INITIALIZER_NP \
  
#  else
#   if __BYTE_ORDER == __LITTLE_ENDIAN
#    define PTHREAD_RWLOCK_WRITER_NONRECURSIVE_INITIALIZER_NP \
  
#   else
#    define PTHREAD_RWLOCK_WRITER_NONRECURSIVE_INITIALIZER_NP \
  
#   endif
#  endif
# endif
#endif  /* Unix98 or XOpen2K */


/* Scheduler inheritance.  */
enum
{
   
#define PTHREAD_EXPLICIT_SCHED  PTHREAD_EXPLICIT_SCHED
};


/* Scope handling.  */
enum
{
   
#define PTHREAD_SCOPE_PROCESS   PTHREAD_SCOPE_PROCESS
};


/* Process shared or private flag.  */
enum
{
   
#define PTHREAD_PROCESS_SHARED  PTHREAD_PROCESS_SHARED
};



/* Conditional variable handling.  */
#define PTHREAD_COND_INITIALIZER 


/* Cleanup buffers */
struct _pthread_cleanup_buffer
{
  void *__routine ;             /* Function to call.  */
  void *__arg;                            /* Its argument.  */
  int __canceltype;                       /* Saved cancellation type. */
  struct _pthread_cleanup_buffer *__prev; /* Chaining of cleanup functions.  */
};

/* Cancellation */
enum
{
   
#define PTHREAD_CANCEL_DISABLE  PTHREAD_CANCEL_DISABLE
};
enum
{
   
#define PTHREAD_CANCEL_ASYNCHRONOUS	PTHREAD_CANCEL_ASYNCHRONOUS
};
#define PTHREAD_CANCELED 


/* Single execution handling.  */
#define PTHREAD_ONCE_INIT 0


#ifdef __USE_XOPEN2K
/* Value returned by 'pthread_barrier_wait' for one of the threads after
   the required number of threads have called this function.
   -1 is distinct from 0 and all errno constants */
# define PTHREAD_BARRIER_SERIAL_THREAD -1
#endif


__BEGIN_DECLS

/* Create a new thread, starting with execution of START-ROUTINE
   getting passed ARG.  Creation attributed come from ATTR.  The new
   handle is stored in *NEWTHREAD.  */
void  pthread_create (
			   
			   void * ,
			   void *__restrict __arg) __THROWNL __nonnull ();

/* Terminate calling thread.

   The registered cleanup handlers are called via exception handling
   so we cannot mark this function with __THROW.*/


/* Make calling thread wait for termination of the thread TH.  The
   exit status of the thread is stored in *THREAD_RETURN, if THREAD_RETURN
   is not NULL.

   This function is a cancellation point and therefore not marked with
   __THROW.  */
extern int pthread_join ;

#ifdef __USE_GNU
/* Check whether thread TH has terminated.  If yes return the status of
   the thread in *THREAD_RETURN, if THREAD_RETURN is not NULL.  */
extern int pthread_tryjoin_np (pthread_t __th, void **__thread_return) __THROW;

/* Make calling thread wait for termination of the thread TH, but only
   until TIMEOUT.  The exit status of the thread is stored in
   *THREAD_RETURN, if THREAD_RETURN is not NULL.

   This function is a cancellation point and therefore not marked with
   __THROW.  */
extern int pthread_timedjoin_np ;
#endif

/* Indicate that the thread TH is never to be joined with PTHREAD_JOIN.
   The resources of TH will therefore be freed immediately when it
   terminates, instead of waiting for another thread to perform PTHREAD_JOIN
   on it.  */
void  pthread_detach (void) __THROW;


/* Obtain the identifier of the current thread.  */
void  pthread_self (void) __THROW __attribute__ ((__const__));

/* Compare two thread identifiers.  */
void  pthread_equal ( void)
  __THROW __attribute__ ((__const__));


/* Thread attribute handling.  */

/* Initialize thread attribute *ATTR with default attributes
   .  */


/* Destroy thread attribute *ATTR.  */


/* Get detach state attribute.  */


/* Set detach state attribute.  */



/* Get the size of the guard area created for stack overflow protection.  */


/* Set the size of the guard area created for stack overflow protection.  */



/* Return in *PARAM the scheduling parameters of *ATTR.  */


/* Set scheduling parameters  in *ATTR according to PARAM.  */


/* Return in *POLICY the scheduling policy of *ATTR.  */


/* Set scheduling policy in *ATTR according to POLICY.  */


/* Return in *INHERIT the scheduling inheritance mode of *ATTR.  */


/* Set scheduling inheritance mode in *ATTR according to INHERIT.  */



/* Return in *SCOPE the scheduling contention scope of *ATTR.  */


/* Set scheduling contention scope in *ATTR according to SCOPE.  */


/* Return the previously set address for the stack.  */


/* Set the starting address of the stack of the thread to be created.
   Depending on whether the stack grows up or down the value must either
   be higher or lower than all the address in the memory block.  The
   minimal size of the block must be PTHREAD_STACK_MIN.  */
void  pthread_attr_setstackaddr (
				      void)
     __THROW __nonnull () __attribute_deprecated__;

/* Return the currently used minimal stack size.  */


/* Add information about the minimum stack size needed for the thread
   to be started.  This size must never be less than PTHREAD_STACK_MIN
   and must also not exceed the system limits.  */


#ifdef __USE_XOPEN2K
/* Return the previously set address for the stack.  */


/* The following two interfaces are intended to replace the last two.  They
   require setting the address as well as the size since only setting the
   address will make the implementation on some architectures impossible.  */

#endif

#ifdef __USE_GNU
/* Thread created with attribute ATTR will be limited to run only on
   the processors represented in CPUSET.  */
extern int pthread_attr_setaffinity_np (pthread_attr_t *__attr,
					size_t __cpusetsize,
					const cpu_set_t *__cpuset)
     __THROW __nonnull ();

/* Get bit set in CPUSET representing the processors threads created with
   ATTR can run on.  */
extern int pthread_attr_getaffinity_np (const pthread_attr_t *__attr,
					size_t __cpusetsize,
					cpu_set_t *__cpuset)
     __THROW __nonnull ();

/* Get the default attributes used by pthread_create in this process.  */
extern int pthread_getattr_default_np (pthread_attr_t *__attr)
     __THROW __nonnull ();

/* Set the default attributes to be used by pthread_create in this
   process.  */
extern int pthread_setattr_default_np (const pthread_attr_t *__attr)
     __THROW __nonnull ();

/* Initialize thread attribute *ATTR with attributes corresponding to the
   already running thread TH.  It shall be called on uninitialized ATTR
   and destroyed with pthread_attr_destroy when no longer needed.  */
extern int pthread_getattr_np (pthread_t __th, pthread_attr_t *__attr)
     __THROW __nonnull ();
#endif


/* Functions for scheduling control.  */

/* Set the scheduling parameters for TARGET_THREAD according to POLICY
   and *PARAM.  */


/* Return in *POLICY and *PARAM the scheduling parameters for TARGET_THREAD. */


/* Set the scheduling priority for TARGET_THREAD.  */



#ifdef __USE_GNU
/* Get thread name visible in the kernel and its interfaces.  */
extern int pthread_getname_np (pthread_t __target_thread, char *__buf,
			       size_t __buflen)
     __THROW __nonnull ();

/* Set thread name visible in the kernel and its interfaces.  */
extern int pthread_setname_np (pthread_t __target_thread, const char *__name)
     __THROW __nonnull ();
#endif


#ifdef __USE_UNIX98
/* Determine level of concurrency.  */
extern int pthread_getconcurrency (void) __THROW;

/* Set new concurrency level to LEVEL.  */
extern int pthread_setconcurrency (int __level) __THROW;
#endif

#ifdef __USE_GNU
/* Yield the processor to another thread or process.
   This function is similar to the POSIX `sched_yield' function but
   might be differently implemented in the case of a m-on-n thread
   implementation.  */
extern int pthread_yield (void) __THROW;


/* Limit specified thread TH to run only on the processors represented
   in CPUSET.  */
extern int pthread_setaffinity_np (pthread_t __th, size_t __cpusetsize,
				   const cpu_set_t *__cpuset)
     __THROW __nonnull ();

/* Get bit set in CPUSET representing the processors TH can run on.  */
extern int pthread_getaffinity_np (pthread_t __th, size_t __cpusetsize,
				   cpu_set_t *__cpuset)
     __THROW __nonnull ();
#endif


/* Functions for handling initialization.  */

/* Guarantee that the initialization function INIT_ROUTINE will be called
   only once, even if pthread_once is executed several times with the
   same ONCE_CONTROL argument. ONCE_CONTROL must point to a static or
   extern variable initialized to PTHREAD_ONCE_INIT.

   The initialization functions might throw exception which is why
   this function is not marked with __THROW.  */
extern int pthread_once (pthread_once_t *__once_control,
			 void  (void)) __nonnull ();


/* Functions for handling cancellation.

   Note that these functions are explicitly not marked to not throw an
   exception in C++ code.  If cancellation is implemented by unwinding
   this is necessary to have the compiler generate the unwind information.  */

/* Set cancelability state of current thread to STATE, returning old
   state in *OLDSTATE if OLDSTATE is not NULL.  */
extern int pthread_setcancelstate ;

/* Set cancellation state of current thread to TYPE, returning the old
   type in *OLDTYPE if OLDTYPE is not NULL.  */
extern int pthread_setcanceltype ;

/* Cancel THREAD immediately or at the next possibility.  */
extern int pthread_cancel ;

/* Test for pending cancellation for the current thread and terminate
   the thread as per pthread_exit if it has been
   cancelled.  */



/* Cancellation handling with integration into exception handling.  */

typedef struct
{
  struct
  {
    __jmp_buf __cancel_jmp_buf;
    int __mask_was_saved;
  } __cancel_jmp_buf[1];
  void *__pad[4];
} __pthread_unwind_buf_t __attribute__ ((__aligned__));

/* No special attributes by default.  */
#ifndef __cleanup_fct_attribute
# define __cleanup_fct_attribute
#endif


/* Structure to hold the cleanup handler information.  */
struct __pthread_cleanup_frame
;

#if defined __GNUC__ && defined __EXCEPTIONS
# ifdef __cplusplus
/* Class to handle cancellation handler invocation.  */
class __pthread_cleanup_class
{
  void (*__cancel_routine) (void *);
  void *__cancel_arg;
  int __do_it;
  int __cancel_type;

 public:
  __pthread_cleanup_class (void (*__fct) (void *), void *__arg)
    : __cancel_routine (__fct), __cancel_arg (__arg), __do_it (1) { }
  ~__pthread_cleanup_class () ;
  void __setdoit (int __newval) ;
  void __defer () ;
  void __restore () const ;
};

/* Install a cleanup handler: ROUTINE will be called with arguments ARG
   when the thread is canceled or calls pthread_exit.  ROUTINE will also
   be called with arguments ARG when the matching pthread_cleanup_pop
   is executed with non-zero EXECUTE argument.

   pthread_cleanup_push and pthread_cleanup_pop are macros and must always
   be used in matching pairs at the same nesting level of braces.  */
#  define pthread_cleanup_push \
  do 									      \
    __pthread_cleanup_class __clframe 

/* Remove a cleanup handler installed by the matching pthread_cleanup_push.
   If EXECUTE is non-zero, the handler function is called. */
#  define pthread_cleanup_pop \
    __clframe.__setdoit ;					      \
   while 

#  ifdef __USE_GNU
/* Install a cleanup handler as pthread_cleanup_push does, but also
   saves the current cancellation type and sets it to deferred
   cancellation.  */
#   define pthread_cleanup_push_defer_np \
  do 									      \
    __pthread_cleanup_class __clframe ;			      \
    __clframe.__defer 

/* Remove a cleanup handler as pthread_cleanup_pop does, but also
   restores the cancellation type that was in effect when the matching
   pthread_cleanup_push_defer was called.  */
#   define pthread_cleanup_pop_restore_np \
    __clframe.__restore ;						      \
    __clframe.__setdoit ;					      \
   while 
#  endif
# else
/* Function called to call the cleanup handler.  As an extern inline
   function the compiler is free to decide inlining the change when
   needed or fall back on the copy which must exist somewhere
   else.  */
__extern_inline void
__pthread_cleanup_routine 


/* Install a cleanup handler: ROUTINE will be called with arguments ARG
   when the thread is canceled or calls pthread_exit.  ROUTINE will also
   be called with arguments ARG when the matching pthread_cleanup_pop
   is executed with non-zero EXECUTE argument.

   pthread_cleanup_push and pthread_cleanup_pop are macros and must always
   be used in matching pairs at the same nesting level of braces.  */
#  define pthread_cleanup_push \
  do 									      \
    struct __pthread_cleanup_frame __clframe				      \
      __attribute__ 		      \
      = ;

/* Remove a cleanup handler installed by the matching pthread_cleanup_push.
   If EXECUTE is non-zero, the handler function is called. */
#  define pthread_cleanup_pop \
    __clframe.__do_it = ;					      \
   while 

#  ifdef __USE_GNU
/* Install a cleanup handler as pthread_cleanup_push does, but also
   saves the current cancellation type and sets it to deferred
   cancellation.  */
#   define pthread_cleanup_push_defer_np \
  do 									      \
    struct __pthread_cleanup_frame __clframe				      \
      __attribute__ 		      \
      = ;						      \
     pthread_setcanceltype 

/* Remove a cleanup handler as pthread_cleanup_pop does, but also
   restores the cancellation type that was in effect when the matching
   pthread_cleanup_push_defer was called.  */
#   define pthread_cleanup_pop_restore_np \
     pthread_setcanceltype ;	      \
    __clframe.__do_it = ;					      \
   while 
#  endif
# endif
#else
/* Install a cleanup handler: ROUTINE will be called with arguments ARG
   when the thread is canceled or calls pthread_exit.  ROUTINE will also
   be called with arguments ARG when the matching pthread_cleanup_pop
   is executed with non-zero EXECUTE argument.

   pthread_cleanup_push and pthread_cleanup_pop are macros and must always
   be used in matching pairs at the same nesting level of braces.  */
# define pthread_cleanup_push \
  do 									      \
    __pthread_unwind_buf_t __cancel_buf;				      \
    void   = ;			      \
    void *__cancel_arg = ;						      \
    int __not_first_call = __sigsetjmp ;    \
    if 				      \
      									      \
	__cancel_routine ;				      \
	__pthread_unwind_next ;				      \
	/* NOTREACHED */						      \
      									      \
									      \
    __pthread_register_cancel ;				      \
    do 
extern void __pthread_register_cancel 
     __cleanup_fct_attribute;

/* Remove a cleanup handler installed by the matching pthread_cleanup_push.
   If EXECUTE is non-zero, the handler function is called. */
# define pthread_cleanup_pop \
      do  while ;/* Empty to allow label before pthread_cleanup_pop.  */\
     while ;							      \
    __pthread_unregister_cancel ;			      \
    if 							      \
      __cancel_routine ;					      \
   while 
extern void __pthread_unregister_cancel 
  __cleanup_fct_attribute;

# ifdef __USE_GNU
/* Install a cleanup handler as pthread_cleanup_push does, but also
   saves the current cancellation type and sets it to deferred
   cancellation.  */
#  define pthread_cleanup_push_defer_np \
  do 									      \
    __pthread_unwind_buf_t __cancel_buf;				      \
    void   = ;			      \
    void *__cancel_arg = ;						      \
    int __not_first_call = __sigsetjmp ;    \
    if 				      \
      									      \
	__cancel_routine ;				      \
	__pthread_unwind_next ;				      \
	/* NOTREACHED */						      \
      									      \
									      \
    __pthread_register_cancel_defer ;			      \
    do 
extern void __pthread_register_cancel_defer 
     __cleanup_fct_attribute;

/* Remove a cleanup handler as pthread_cleanup_pop does, but also
   restores the cancellation type that was in effect when the matching
   pthread_cleanup_push_defer was called.  */
#  define pthread_cleanup_pop_restore_np \
      do  while ;/* Empty to allow label before pthread_cleanup_pop.  */\
     while ;							      \
    __pthread_unregister_cancel_restore ;		      \
    if 							      \
      __cancel_routine ;					      \
   while 
extern void __pthread_unregister_cancel_restore 
  __cleanup_fct_attribute;
# endif

/* Internal interface to initiate cleanup.  */
extern void __pthread_unwind_next 
     __cleanup_fct_attribute __attribute__ 
# ifndef SHARED
     __attribute__ 
# endif
     ;
#endif

/* Function used in the macros.  */
struct __jmp_buf_tag;



/* Mutex handling.  */

/* Initialize a mutex.  */
void  pthread_mutex_init (
			       void)
     __THROW __nonnull ();

/* Destroy a mutex.  */
extern int pthread_mutex_destroy (pthread_mutex_t *__mutex)
     __THROW __nonnull ();

/* Try locking a mutex.  */
extern int pthread_mutex_trylock (pthread_mutex_t *__mutex)
     __THROWNL __nonnull ();

/* Lock a mutex.  */
extern int pthread_mutex_lock (pthread_mutex_t *__mutex)
     __THROWNL __nonnull ();

#ifdef __USE_XOPEN2K
/* Wait until lock becomes available, or specified time passes. */
extern int pthread_mutex_timedlock (pthread_mutex_t *__restrict __mutex,
				    const struct timespec *__restrict
				    __abstime) __THROWNL __nonnull ();
#endif

/* Unlock a mutex.  */
extern int pthread_mutex_unlock (pthread_mutex_t *__mutex)
     __THROWNL __nonnull ();


/* Get the priority ceiling of MUTEX.  */


/* Set the priority ceiling of MUTEX to PRIOCEILING, return old
   priority ceiling value in *OLD_CEILING.  */



#ifdef __USE_XOPEN2K8
/* Declare the state protected by MUTEX as consistent.  */

# ifdef __USE_GNU
extern int pthread_mutex_consistent_np (pthread_mutex_t *__mutex)
     __THROW __nonnull ();
# endif
#endif


/* Functions for handling mutex attributes.  */

/* Initialize mutex attribute object ATTR with default attributes
   .  */
void  pthread_mutexattr_init (void)
     __THROW __nonnull ();

/* Destroy mutex attribute object ATTR.  */
void  pthread_mutexattr_destroy (void)
     __THROW __nonnull ();

/* Get the process-shared flag of the mutex attribute ATTR.  */


/* Set the process-shared flag of the mutex attribute ATTR.  */


#if defined __USE_UNIX98 || defined __USE_XOPEN2K8
/* Return in *KIND the mutex kind attribute in *ATTR.  */


/* Set the mutex kind attribute in *ATTR to KIND either PTHREAD_MUTEX_NORMAL,
   PTHREAD_MUTEX_RECURSIVE, PTHREAD_MUTEX_ERRORCHECK, or
   PTHREAD_MUTEX_DEFAULT.  */
void  pthread_mutexattr_settype ( void)
     __THROW __nonnull ();
#endif

/* Return in *PROTOCOL the mutex protocol attribute in *ATTR.  */


/* Set the mutex protocol attribute in *ATTR to PROTOCOL .  */


/* Return in *PRIOCEILING the mutex prioceiling attribute in *ATTR.  */


/* Set the mutex prioceiling attribute in *ATTR to PRIOCEILING.  */


#ifdef __USE_XOPEN2K
/* Get the robustness flag of the mutex attribute ATTR.  */

# ifdef __USE_GNU
extern int pthread_mutexattr_getrobust_np (const pthread_mutexattr_t *__attr,
					   int *__robustness)
     __THROW __nonnull ();
# endif

/* Set the robustness flag of the mutex attribute ATTR.  */

# ifdef __USE_GNU
extern int pthread_mutexattr_setrobust_np (pthread_mutexattr_t *__attr,
					   int __robustness)
     __THROW __nonnull ();
# endif
#endif


#if defined __USE_UNIX98 || defined __USE_XOPEN2K
/* Functions for handling read-write locks.  */

/* Initialize read-write lock RWLOCK using attributes ATTR, or use
   the default values if later is NULL.  */


/* Destroy read-write lock RWLOCK.  */


/* Acquire read lock for RWLOCK.  */


/* Try to acquire read lock for RWLOCK.  */


# ifdef __USE_XOPEN2K
/* Try to acquire read lock for RWLOCK or return after specfied time.  */

# endif

/* Acquire write lock for RWLOCK.  */


/* Try to acquire write lock for RWLOCK.  */


# ifdef __USE_XOPEN2K
/* Try to acquire write lock for RWLOCK or return after specfied time.  */

# endif

/* Unlock RWLOCK.  */



/* Functions for handling read-write lock attributes.  */

/* Initialize attribute object ATTR with default values.  */


/* Destroy attribute object ATTR.  */


/* Return current setting of process-shared attribute of ATTR in PSHARED.  */


/* Set process-shared attribute of ATTR to PSHARED.  */


/* Return current setting of reader/writer preference.  */


/* Set reader/write preference.  */

#endif


/* Functions for handling conditional variables.  */

/* Initialize condition variable COND using attributes ATTR, or use
   the default values if later is NULL.  */
void  pthread_cond_init (
			      void)
     __THROW __nonnull ();

/* Destroy condition variable COND.  */
void  pthread_cond_destroy (void)
     __THROW __nonnull ();

/* Wake up one thread waiting for condition variable COND.  */
void  pthread_cond_signal (void)
     __THROWNL __nonnull ();

/* Wake up all threads waiting for condition variables COND.  */
void  pthread_cond_broadcast (void)
     __THROWNL __nonnull ();

/* Wait for condition variable COND to be signaled or broadcast.
   MUTEX is assumed to be locked before.

   This function is a cancellation point and therefore not marked with
   __THROW.  */
extern int pthread_cond_wait 
     __nonnull ();

/* Wait for condition variable COND to be signaled or broadcast until
   ABSTIME.  MUTEX is assumed to be locked before.  ABSTIME is an
   absolute time specification; zero is the beginning of the epoch
   .

   This function is a cancellation point and therefore not marked with
   __THROW.  */
void  pthread_cond_timedwait (
				   
				   void)
     __nonnull ();

/* Functions for handling condition variable attributes.  */

/* Initialize condition variable attribute ATTR.  */


/* Destroy condition variable attribute ATTR.  */


/* Get the process-shared flag of the condition variable attribute ATTR.  */


/* Set the process-shared flag of the condition variable attribute ATTR.  */


#ifdef __USE_XOPEN2K
/* Get the clock selected for the condition variable attribute ATTR.  */


/* Set the clock selected for the condition variable attribute ATTR.  */

#endif


#ifdef __USE_XOPEN2K
/* Functions to handle spinlocks.  */

/* Initialize the spinlock LOCK.  If PSHARED is nonzero the spinlock can
   be shared between different processes.  */


/* Destroy the spinlock LOCK.  */


/* Wait until spinlock LOCK is retrieved.  */


/* Try to lock spinlock LOCK.  */


/* Release spinlock LOCK.  */



/* Functions to handle barriers.  */

/* Initialize BARRIER with the attributes in ATTR.  The barrier is
   opened when COUNT waiters arrived.  */


/* Destroy a previously dynamically initialized barrier BARRIER.  */


/* Wait on barrier BARRIER.  */



/* Initialize barrier attribute ATTR.  */


/* Destroy previously dynamically initialized barrier attribute ATTR.  */


/* Get the process-shared flag of the barrier attribute ATTR.  */


/* Set the process-shared flag of the barrier attribute ATTR.  */

#endif


/* Functions for handling thread-specific data.  */

/* Create a key value identifying a location in the thread-specific
   data area.  Each thread maintains a distinct thread-specific data
   area.  DESTR_FUNCTION, if non-NULL, is called with the value
   associated to that key when the key is destroyed.
   DESTR_FUNCTION is not called if the value associated is NULL when
   the key is destroyed.  */
void  pthread_key_create (
			       void  (void *))
     __THROW __nonnull ();

/* Destroy KEY.  */
void  pthread_key_delete (void) __THROW;

/* Return current value of the thread-specific data slot identified by KEY.  */
void pthread_getspecific (void) __THROW;

/* Store POINTER in the thread-specific data slot identified by KEY. */
void  pthread_setspecific (
				void) __THROW ;


#ifdef __USE_XOPEN2K
/* Get ID of CPU-time clock for thread THREAD_ID.  */

#endif


/* Install handlers to be called when a new process is created with FORK.
   The PREPARE handler is called in the parent process just before performing
   FORK. The PARENT handler is called in the parent process just after FORK.
   The CHILD handler is called in the child process.  Each of the three
   handlers can be NULL, meaning that no handler needs to be called at that
   point.
   PTHREAD_ATFORK can be called several times, in which case the PREPARE
   handlers are called in LIFO order , and the PARENT and CHILD handlers are called
   in FIFO .  */




#ifdef __USE_EXTERN_INLINES
/* Optimizations.  */
__extern_inline int
__NTH pthread_equal pthread_t __thread1, pthread_t __thread2
;
#endif

__END_DECLS

#endif	/* pthread.h */
 