 //  -*- C++ -*-

// Copyright  2010-2015 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or 
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Under Section 7 of GPL version 3, you are granted additional
// permissions described in the GCC Runtime Library Exception, version
// 3.1, as published by the Free Software Foundation.

// You should have received a copy of the GNU General Public License and
// a copy of the GCC Runtime Library Exception along with this program;
// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
// .

/** @file bits/range_access.h
 *  This is an internal header file, included by other library headers.
 *  Do not attempt to use it directly. @headername
 */

#ifndef _GLIBCXX_RANGE_ACCESS_H
#define _GLIBCXX_RANGE_ACCESS_H 1

#pragma GCC system_header

#if __cplusplus >= 201103L
#pragma "/home/regehr/z/compiler-install/gcc-r231221-install/include/c++/6.0.0/initializer_list"
#include "initializer_list"
namespace std _GLIBCXX_VISIBILITY(default)
{
_GLIBCXX_BEGIN_NAMESPACE_VERSION

  /**
   *  @brief  Return an iterator pointing to the first element of
   *          the container.
   *  @param  __cont  Container.
   */
  template<typename _Container>
    inline auto
    begin(_Container& __cont) -> decltype(__cont.begin)
    ;

  /**
   *  @brief  Return an iterator pointing to the first element of
   *          the const container.
   *  @param  __cont  Container.
   */
  template<typename _Container>
    inline auto
    begin(const _Container& __cont) -> decltype(__cont.begin)
    ;

  /**
   *  @brief  Return an iterator pointing to one past the last element of
   *          the container.
   *  @param  __cont  Container.
   */
  template<typename _Container>
    inline auto
    end(_Container& __cont) -> decltype(__cont.end)
    ;

  /**
   *  @brief  Return an iterator pointing to one past the last element of
   *          the const container.
   *  @param  __cont  Container.
   */
  template<typename _Container>
    inline auto
    end(const _Container& __cont) -> decltype(__cont.end)
    ;

  /**
   *  @brief  Return an iterator pointing to the first element of the array.
   *  @param  __arr  Array.
   */
  template<typename _Tp, size_t _Nm>
    inline _GLIBCXX14_CONSTEXPR _Tp*
    begin(_Tp [_Nm])
    ;

  /**
   *  @brief  Return an iterator pointing to one past the last element
   *          of the array.
   *  @param  __arr  Array.
   */
  template<typename _Tp, size_t _Nm>
    inline _GLIBCXX14_CONSTEXPR _Tp*
    end(_Tp [_Nm])
    ;

#if __cplusplus >= 201402L

  template class valarray;
  // These overloads must be declared for cbegin and cend to use them.
  template _Tp* begin;
  template const _Tp* begin;
  template _Tp* end;
  template const _Tp* end;

  /**
   *  @brief  Return an iterator pointing to the first element of
   *          the const container.
   *  @param  __cont  Container.
   */
  template
    inline constexpr auto
    cbegin noexcept
      -> decltype
    

  /**
   *  @brief  Return an iterator pointing to one past the last element of
   *          the const container.
   *  @param  __cont  Container.
   */
  template
    inline constexpr auto
    cend noexcept
      -> decltype
    

  /**
   *  @brief  Return a reverse iterator pointing to the last element of
   *          the container.
   *  @param  __cont  Container.
   */
  template
    inline auto
    rbegin -> decltype
    

  /**
   *  @brief  Return a reverse iterator pointing to the last element of
   *          the const container.
   *  @param  __cont  Container.
   */
  template
    inline auto
    rbegin -> decltype
    

  /**
   *  @brief  Return a reverse iterator pointing one past the first element of
   *          the container.
   *  @param  __cont  Container.
   */
  template
    inline auto
    rend -> decltype
    

  /**
   *  @brief  Return a reverse iterator pointing one past the first element of
   *          the const container.
   *  @param  __cont  Container.
   */
  template
    inline auto
    rend -> decltype
    

  /**
   *  @brief  Return a reverse iterator pointing to the last element of
   *          the array.
   *  @param  __arr  Array.
   */
  template
    inline reverse_iterator
    rbegin
    

  /**
   *  @brief  Return a reverse iterator pointing one past the first element of
   *          the array.
   *  @param  __arr  Array.
   */
  template
    inline reverse_iterator
    rend
    

  /**
   *  @brief  Return a reverse iterator pointing to the last element of
   *          the initializer_list.
   *  @param  __il  initializer_list.
   */
  template
    inline reverse_iterator
    rbegin
    

  /**
   *  @brief  Return a reverse iterator pointing one past the first element of
   *          the initializer_list.
   *  @param  __il  initializer_list.
   */
  template
    inline reverse_iterator
    rend
    

  /**
   *  @brief  Return a reverse iterator pointing to the last element of
   *          the const container.
   *  @param  __cont  Container.
   */
  template
    inline auto
    crbegin -> decltype
    

  /**
   *  @brief  Return a reverse iterator pointing one past the first element of
   *          the const container.
   *  @param  __cont  Container.
   */
  template
    inline auto
    crend -> decltype
    

#endif // C++14

#if __cplusplus > 201402L
#define __cpp_lib_nonmember_container_access 201411

  /**
   *  @brief  Return the size of a container.
   *  @param  __cont  Container.
   */
  template 
    constexpr auto
    size -> decltype
    

  /**
   *  @brief  Return the size of an array.
   *  @param  __array  Array.
   */
  template 
    constexpr size_t
    size noexcept
    

  /**
   *  @brief  Return whether a container is empty.
   *  @param  __cont  Container.
   */
  template 
    constexpr auto
    empty -> decltype
    

  /**
   *  @brief  Return whether an array is empty .
   *  @param  __array  Container.
   */
  template 
    constexpr bool
    empty noexcept
    

  /**
   *  @brief  Return whether an initializer_list is empty.
   *  @param  __il  Initializer list.
   */
  template 
    constexpr bool
    empty noexcept
    

  /**
   *  @brief  Return the data pointer of a container.
   *  @param  __cont  Container.
   */
  template 
    constexpr auto
    data -> decltype
    

  /**
   *  @brief  Return the data pointer of a const container.
   *  @param  __cont  Container.
   */
  template 
    constexpr auto
    data -> decltype
    

  /**
   *  @brief  Return the data pointer of an array.
   *  @param  __array  Array.
   */
  template 
    constexpr _Tp*
    data noexcept
    

  /**
   *  @brief  Return the data pointer of an initializer list.
   *  @param  __il  Initializer list.
   */
  template 
    constexpr const _Tp*
    data noexcept
    

#endif // C++17

_GLIBCXX_END_NAMESPACE_VERSION
} // namespace

#endif // C++11

#endif // _GLIBCXX_RANGE_ACCESS_H
 