 // Pair implementation -*- C++ -*-

// Copyright  2001-2015 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 3, or 
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Under Section 7 of GPL version 3, you are granted additional
// permissions described in the GCC Runtime Library Exception, version
// 3.1, as published by the Free Software Foundation.

// You should have received a copy of the GNU General Public License and
// a copy of the GCC Runtime Library Exception along with this program;
// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
// .

/*
 *
 * Copyright  1994
 * Hewlett-Packard Company
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Hewlett-Packard Company makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 *
 *
 * Copyright  1996,1997
 * Silicon Graphics Computer Systems, Inc.
 *
 * Permission to use, copy, modify, distribute and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear
 * in supporting documentation.  Silicon Graphics makes no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 */

/** @file bits/stl_pair.h
 *  This is an internal header file, included by other library headers.
 *  Do not attempt to use it directly. @headername
 */

#ifndef _STL_PAIR_H
#define _STL_PAIR_H 1

#pragma "/home/regehr/z/compiler-install/gcc-r231221-install/include/c++/6.0.0/bits/move.h"
#include "move.h"

#if __cplusplus >= 201103L
#pragma "/home/regehr/z/compiler-install/gcc-r231221-install/include/c++/6.0.0/type_traits"
#include "type_traits"
#endif

namespace std _GLIBCXX_VISIBILITY(default)
{
_GLIBCXX_BEGIN_NAMESPACE_VERSION

  /**
   *  @addtogroup utilities
   *  @
   */

#if __cplusplus >= 201103L
  /// piecewise_construct_t
  struct piecewise_construct_t { explicit piecewise_construct_t() = default; };

  /// piecewise_construct
  constexpr piecewise_construct_t piecewise_construct = piecewise_construct_t();

  // Forward declarations.
  template<typename...>
    class tuple;

  template<std::size_t...>
    struct _Index_tuple;

  // Concept utility functions, reused in conditionally-explicit
  // constructors.
  template <typename _T1, typename _T2, typename _U1, typename _U2>
  constexpr bool _ConstructiblePair()
  {
    return __and_<is_constructible<_T1, const _U1&>,
		  is_constructible<_T2, const _U2&>>::value;
  }

  template <typename _T1, typename _T2, typename _U1, typename _U2>
  constexpr bool _ImplicitlyConvertiblePair()
  {
    return __and_<is_convertible<const _U1&, _T1>,
		  is_convertible<const _U2&, _T2>>::value;
  }

  template <typename _T1, typename _T2, typename _U1, typename _U2>
  constexpr bool _MoveConstructiblePair
  ;

  template <typename _T1, typename _T2, typename _U1, typename _U2>
  constexpr bool _ImplicitlyMoveConvertiblePair
  ;


#endif

 /**
   *  @brief Struct holding two objects of arbitrary type.
   *
   *  @tparam _T1  Type of first object.
   *  @tparam _T2  Type of second object.
   */
  template<typename _T1, typename _T2>
    struct pair
    {
      typedef _T1 first_type;    /// @c first_type is the first bound type
      typedef _T2 second_type;   /// @c second_type is the second bound type

      _T1 first;                 /// @c first is a copy of the first object
      _T2 second;                /// @c second is a copy of the second object

      // _GLIBCXX_RESOLVE_LIB_DEFECTS
      // 265.  std::pair::pair effects overly restrictive
      /** The default constructor creates @c first and @c second using their
       *  respective default constructors.  */
#if __cplusplus >= 201103L
      template <typename _U1 = _T1,
                typename _U2 = _T2,
                typename enable_if<__and_<
                                     __is_implicitly_default_constructible<_U1>,
                                     __is_implicitly_default_constructible<_U2>>
                                   ::value, bool>::type = true>
#endif
      _GLIBCXX_CONSTEXPR pair()
      : first(), second() { }

#if __cplusplus >= 201103L
      template <typename _U1 = _T1,
                typename _U2 = _T2,
                typename enable_if<__and_<
                       is_default_constructible<_U1>,
                       is_default_constructible<_U2>,
                       __not_<
                         __and_<__is_implicitly_default_constructible<_U1>,
                                __is_implicitly_default_constructible<_U2>>>>
                                   ::value, bool>::type = false>
      explicit constexpr pair()
      : first(), second() { }
#endif

      /** Two objects may be passed to a @c pair constructor to be copied.  */
#if __cplusplus < 201103L
      pair
      : first, second 
#else
      template<typename _U1 = _T1, typename _U2=_T2, typename
                enable_if<_ConstructiblePair
                         && _ImplicitlyConvertiblePair<_T1, _T2, _U1, _U2>,
                         bool>::type=true>
      constexpr pair(const _T1& __a, const _T2& __b)
      : first(__a), second(__b) { }

       template<typename _U1 = _T1, typename _U2=_T2, typename
	       enable_if<_ConstructiblePair
                         && !_ImplicitlyConvertiblePair<_T1, _T2, _U1, _U2>,
                         bool>::type=false>
      explicit constexpr pair(const _T1& __a, const _T2& __b)
      : first(__a), second(__b) { }
#endif

      /** There is also a templated copy ctor for the @c pair class itself.  */
#if __cplusplus < 201103L
      template
	pair
	: first, second 
#else
      template<typename _U1, typename _U2, typename
	       enable_if<_ConstructiblePair
                         && _ImplicitlyConvertiblePair<_T1, _T2, _U1, _U2>,
                         bool>::type=true>
        constexpr pair(const pair& __p)
        : first(__p.first), second(__p.second) { }

      template<typename _U1, typename _U2, typename
               enable_if<_ConstructiblePair
                         && !_ImplicitlyConvertiblePair<_T1, _T2, _U1, _U2>,
                         bool>::type=false>
	explicit constexpr pair(const pair& __p)
	: first(__p.first), second(__p.second) { }

      constexpr pair(const pair&) = default;
      constexpr pair(pair&&) = default;

      // DR 811.
      template<typename _U1, typename
               enable_if<_ConstructiblePair
                         && _MoveConstructiblePair<_T1, _T2, _U1, _T2>
                         && _ImplicitlyConvertiblePair
                         && _ImplicitlyMoveConvertiblePair<_T1, _T2,
							  _U1, _T2>,
                         bool>::type=true>
       constexpr pair(_U1&& __x, const _T2& __y)
       : first(std::forward), second(__y) { }

      template<typename _U1, typename
               enable_if<_ConstructiblePair
                         && _MoveConstructiblePair<_T1, _T2, _U1, _T2>
                         && !_ImplicitlyConvertiblePair<_T2, _T2, _T2, _T2>
                             || !_ImplicitlyMoveConvertiblePair<_T1, _T2,
                                                                _U1, _T2>,
                         bool>::type=false>
       explicit constexpr pair(_U1&& __x, const _T2& __y)
       : first(std::forward), second(__y) { }

      template<typename _U2, typename
               enable_if<_ConstructiblePair
                         && _MoveConstructiblePair<_T1, _T2, _T1, _U2>
                         && _ImplicitlyConvertiblePair
                         && _ImplicitlyMoveConvertiblePair<_T1, _T2,
                                                           _T1, _U2>,
                         bool>::type=true>
       constexpr pair(const _T1& __x, _U2&& __y)
       : first(__x), second(std::forward) { }

      template<typename _U2, typename
               enable_if<_ConstructiblePair
                         && _MoveConstructiblePair<_T1, _T2, _T1, _U2>
                         && !_ImplicitlyConvertiblePair<_T1, _T1, _T1, _T1>
                             || !_ImplicitlyMoveConvertiblePair<_T1, _T2,
                                                                _T1, _U2>,
                         bool>::type=false>
       explicit pair(const _T1& __x, _U2&& __y)
       : first(__x), second(std::forward) { }

      template<typename _U1, typename _U2, typename
	       enable_if<_MoveConstructiblePair<_T1, _T2, _U1, _U2>
                         && _ImplicitlyMoveConvertiblePair<_T1, _T2,
							   _U1, _U2>,
                         bool>::type=true>
	constexpr pair(_U1&& __x, _U2&& __y)
	: first(std::forward), second(std::forward) { }

      template<typename _U1, typename _U2, typename
	       enable_if<_MoveConstructiblePair<_T1, _T2, _U1, _U2>
                         && !_ImplicitlyMoveConvertiblePair<_T1, _T2,
							    _U1, _U2>,
                         bool>::type=false>
	explicit constexpr pair(_U1&& __x, _U2&& __y)
	: first(std::forward), second(std::forward) { }


      template<typename _U1, typename _U2, typename
	       enable_if<_MoveConstructiblePair<_T1, _T2, _U1, _U2>
                         && _ImplicitlyMoveConvertiblePair<_T1, _T2,
							   _U1, _U2>,
                         bool>::type=true>
	constexpr pair(pair&& __p)
	: first(std::forward),
	  second(std::forward) { }

      template<typename _U1, typename _U2, typename
	       enable_if<_MoveConstructiblePair<_T1, _T2, _U1, _U2>
                         && !_ImplicitlyMoveConvertiblePair<_T1, _T2,
							   _U1, _U2>,
                         bool>::type=false>
	explicit constexpr pair(pair&& __p)
	: first(std::forward),
	  second(std::forward) { }

      template<typename... _Args1, typename... _Args2>
        pair(piecewise_construct_t, tuple<_Args1...>, tuple<_Args2...>);

      pair&
      operator=(const pair& __p)
      ;

      pair&
      operator=(pair&& __p)
      noexcept
      {
	first = std::forward;
	second = std::forward;
	return *this;
      }

      template<typename _U1, typename _U2>
	pair&
	operator=(const pair& __p)
	;

      template<typename _U1, typename _U2>
	pair&
	operator=(pair&& __p)
	{
	  first = std::forward;
	  second = std::forward;
	  return *this;
	}

      void
      swap(pair& __p)
      noexcept
      ;

    private:
      template<typename... _Args1, std::size_t... _Indexes1,
               typename... _Args2, std::size_t... _Indexes2>
        pair(tuple<_Args1...>&, tuple<_Args2...>&,
             _Index_tuple<_Indexes1...>, _Index_tuple<_Indexes2...>);
#endif
    };

  /// Two pairs of the same type are equal iff their members are equal.
  template<typename _T1, typename _T2>
    inline _GLIBCXX_CONSTEXPR bool
    operator==(const pair<_T1, _T2>& __x, const pair<_T1, _T2>& __y)
    ;

  /// 
  template<typename _T1, typename _T2>
    inline _GLIBCXX_CONSTEXPR bool
    operator<(const pair<_T1, _T2>& __x, const pair<_T1, _T2>& __y)
    ;

  /// Uses @c operator== to find the result.
  template<typename _T1, typename _T2>
    inline _GLIBCXX_CONSTEXPR bool
    operator!=(const pair<_T1, _T2>& __x, const pair<_T1, _T2>& __y)
    ;

  /// Uses @c operator< to find the result.
  template<typename _T1, typename _T2>
    inline _GLIBCXX_CONSTEXPR bool
    operator>(const pair<_T1, _T2>& __x, const pair<_T1, _T2>& __y)
    ;

  /// Uses @c operator< to find the result.
  template<typename _T1, typename _T2>
    inline _GLIBCXX_CONSTEXPR bool
    operator<=(const pair<_T1, _T2>& __x, const pair<_T1, _T2>& __y)
    ;

  /// Uses @c operator< to find the result.
  template<typename _T1, typename _T2>
    inline _GLIBCXX_CONSTEXPR bool
    operator>=(const pair<_T1, _T2>& __x, const pair<_T1, _T2>& __y)
    ;

#if __cplusplus >= 201103L
  /// See std::pair::swap.
  // Note:  no std::swap overloads in C++03 mode, this has performance
  //        implications, see, eg, libstdc++/38466.
  template<typename _T1, typename _T2>
    inline void
    swap(pair<_T1, _T2>& __x, pair<_T1, _T2>& __y)
    noexcept
    ;
#endif

  /**
   *  @brief A convenience wrapper for creating a pair from two objects.
   *  @param  __x  The first object.
   *  @param  __y  The second object.
   *  @return   A newly-constructed pair object of the appropriate type.
   *
   *  The standard requires that the objects be passed by reference-to-const,
   *  but LWG issue #181 says they should be passed by const value.  We follow
   *  the LWG by default.
   */
  // _GLIBCXX_RESOLVE_LIB_DEFECTS
  // 181.  make_pair unintended behavior
#if __cplusplus >= 201103L
  // NB: DR 706.
  template<typename _T1, typename _T2>
    constexpr pair<typename __decay_and_strip<_T1>::__type,
                   typename __decay_and_strip<_T2>::__type>
    make_pair(_T1&& __x, _T2&& __y)
    {
      typedef typename __decay_and_strip<_T1>::__type __ds_type1;
      typedef typename __decay_and_strip<_T2>::__type __ds_type2;
      typedef pair<__ds_type1, __ds_type2> 	      __pair_type;
      return __pair_type(std::forward, std::forward);
    }
#else
  template
    inline pair
    make_pair
    
#endif

  /// @

_GLIBCXX_END_NAMESPACE_VERSION
} // namespace std

#endif /* _STL_PAIR_H */
 