 /* Copyright  1995-2014 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or  any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   .  */

#ifndef	_BITS_TIMEX_H
#define	_BITS_TIMEX_H	1

#pragma "/usr/include/x86_64-linux-gnu/bits/types.h"
#include "types.h"

/* These definitions from linux/timex.h as of 2.6.30.  */

struct timex
{
  unsigned int modes;		/* mode selector */
  __syscall_slong_t offset;	/* time offset  */
  __syscall_slong_t freq;	/* frequency offset  */
  __syscall_slong_t maxerror;	/* maximum error  */
  __syscall_slong_t esterror;	/* estimated error  */
  int status;			/* clock command/status */
  __syscall_slong_t constant;	/* pll time constant */
  __syscall_slong_t precision;	/* clock precision   */
  __syscall_slong_t tolerance;	/* clock frequency tolerance   */
  struct timeval time;		/*  */
  __syscall_slong_t tick;	/*  usecs between clock ticks */
  __syscall_slong_t ppsfreq;	/* pps frequency   */
  __syscall_slong_t jitter;	/* pps jitter   */
  int shift;			/* interval duration    */
  __syscall_slong_t stabil;	/* pps stability   */
  __syscall_slong_t jitcnt;	/* jitter limit exceeded  */
  __syscall_slong_t calcnt;	/* calibration intervals  */
  __syscall_slong_t errcnt;	/* calibration errors  */
  __syscall_slong_t stbcnt;	/* stability limit exceeded  */

  int tai;			/* TAI offset  */

  /* ??? */
  int  :32; int  :32; int  :32; int  :32;
  int  :32; int  :32; int  :32; int  :32;
  int  :32; int  :32; int  :32;
};

/* Mode codes  */
#define ADJ_OFFSET		0x0001	/* time offset */
#define ADJ_FREQUENCY		0x0002	/* frequency offset */
#define ADJ_MAXERROR		0x0004	/* maximum time error */
#define ADJ_ESTERROR		0x0008	/* estimated time error */
#define ADJ_STATUS		0x0010	/* clock status */
#define ADJ_TIMECONST		0x0020	/* pll time constant */
#define ADJ_TAI			0x0080	/* set TAI offset */
#define ADJ_MICRO		0x1000	/* select microsecond resolution */
#define ADJ_NANO		0x2000	/* select nanosecond resolution */
#define ADJ_TICK		0x4000	/* tick value */
#define ADJ_OFFSET_SINGLESHOT	0x8001	/* old-fashioned adjtime */
#define ADJ_OFFSET_SS_READ	0xa001	/* read-only adjtime */

/* xntp 3.4 compatibility names */
#define MOD_OFFSET	ADJ_OFFSET
#define MOD_FREQUENCY	ADJ_FREQUENCY
#define MOD_MAXERROR	ADJ_MAXERROR
#define MOD_ESTERROR	ADJ_ESTERROR
#define MOD_STATUS	ADJ_STATUS
#define MOD_TIMECONST	ADJ_TIMECONST
#define MOD_CLKB	ADJ_TICK
#define MOD_CLKA	ADJ_OFFSET_SINGLESHOT /* 0x8000 in original */
#define MOD_TAI		ADJ_TAI
#define MOD_MICRO	ADJ_MICRO
#define MOD_NANO	ADJ_NANO


/* Status codes  */
#define STA_PLL		0x0001	/* enable PLL updates  */
#define STA_PPSFREQ	0x0002	/* enable PPS freq discipline  */
#define STA_PPSTIME	0x0004	/* enable PPS time discipline  */
#define STA_FLL		0x0008	/* select frequency-lock mode  */

#define STA_INS		0x0010	/* insert leap  */
#define STA_DEL		0x0020	/* delete leap  */
#define STA_UNSYNC	0x0040	/* clock unsynchronized  */
#define STA_FREQHOLD	0x0080	/* hold frequency  */

#define STA_PPSSIGNAL	0x0100	/* PPS signal present  */
#define STA_PPSJITTER	0x0200	/* PPS signal jitter exceeded  */
#define STA_PPSWANDER	0x0400	/* PPS signal wander exceeded  */
#define STA_PPSERROR	0x0800	/* PPS signal calibration error  */

#define STA_CLOCKERR	0x1000	/* clock hardware fault  */
#define STA_NANO	0x2000	/* resolution   */
#define STA_MODE	0x4000	/* mode   */
#define STA_CLK		0x8000	/* clock source   */

/* Read-only bits */
#define STA_RONLY STA_PPSSIGNAL | STA_PPSJITTER | STA_PPSWANDER | \
    STA_PPSERROR | STA_CLOCKERR | STA_NANO | STA_MODE | STA_CLK

#endif /* bits/timex.h */
 