 // -*- C++ -*-

// Copyright  2005-2015 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the terms
// of the GNU General Public License as published by the Free Software
// Foundation; either version 3, or  any later
// version.

// This library is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.

// Under Section 7 of GPL version 3, you are granted additional
// permissions described in the GCC Runtime Library Exception, version
// 3.1, as published by the Free Software Foundation.

// You should have received a copy of the GNU General Public License and
// a copy of the GCC Runtime Library Exception along with this program;
// see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
// .

/** @file ext/type_traits.h
 *  This file is a GNU extension to the Standard C++ Library.
 */

#ifndef _EXT_TYPE_TRAITS
#define _EXT_TYPE_TRAITS 1

#pragma GCC system_header

#pragma "/home/regehr/z/compiler-install/gcc-r231221-install/include/c++/6.0.0/x86_64-pc-linux-gnu/bits/c++config.h"
#include "c++config.h"
#pragma "/home/regehr/z/compiler-install/gcc-r231221-install/include/c++/6.0.0/bits/cpp_type_traits.h"
#include "cpp_type_traits.h"

namespace __gnu_cxx _GLIBCXX_VISIBILITY(default)
{
_GLIBCXX_BEGIN_NAMESPACE_VERSION

  // Define a nested type if some predicate holds.
  template<bool, typename>
    struct __enable_if 
    ;

  template<typename _Tp>
    struct __enable_if<true, _Tp>
    ;


  // Conditional expression for types. If true, first, if false, second.
  template<bool _Cond, typename _Iftrue, typename _Iffalse>
    struct __conditional_type
    { typedef _Iftrue __type; };

  template<typename _Iftrue, typename _Iffalse>
    struct __conditional_type<false, _Iftrue, _Iffalse>
    { typedef _Iffalse __type; };


  // Given an integral builtin type, return the corresponding unsigned type.
  template<typename _Tp>
    struct __add_unsigned
    { 
    private:
      typedef __enable_if<std::__is_integer<_Tp>::__value, _Tp> __if_type;
      
    public:
      typedef typename __if_type::__type __type; 
    };

  template<>
    struct __add_unsigned<char>
    ;

  template<>
    struct __add_unsigned<signed char>
    ;

  template<>
    struct __add_unsigned<short>
    ;

  template<>
    struct __add_unsigned<int>
    ;

  template<>
    struct __add_unsigned<long>
    ;

  template<>
    struct __add_unsigned<long long>
    ;

  // Declare but don't define.
  template<>
    struct __add_unsigned<bool>;

  template<>
    struct __add_unsigned<wchar_t>;


  // Given an integral builtin type, return the corresponding signed type.
  template<typename _Tp>
    struct __remove_unsigned
    { 
    private:
      typedef __enable_if<std::__is_integer<_Tp>::__value, _Tp> __if_type;
      
    public:
      typedef typename __if_type::__type __type; 
    };

  template<>
    struct __remove_unsigned<char>
    ;

  template<>
    struct __remove_unsigned<unsigned char>
    ;

  template<>
    struct __remove_unsigned<unsigned short>
    ;

  template<>
    struct __remove_unsigned<unsigned int>
    ;

  template<>
    struct __remove_unsigned<unsigned long>
    ;

  template<>
    struct __remove_unsigned<unsigned long long>
    ;

  // Declare but don't define.
  template<>
    struct __remove_unsigned<bool>;

  template<>
    struct __remove_unsigned<wchar_t>;


  // For use in string and vstring.
  template<typename _Type>
    inline bool
    __is_null_pointer(_Type* __ptr)
    ;

  template<typename _Type>
    inline bool
    __is_null_pointer(_Type)
    ;

#if __cplusplus >= 201103L
  inline bool
  __is_null_pointer(std::nullptr_t)
  ;
#endif

  // For complex and cmath
  template<typename _Tp, bool = std::__is_integer<_Tp>::__value>
    struct __promote
    ;

  // No nested __type member for non-integer non-floating point types,
  // allows this type to be used for SFINAE to constrain overloads in
  //  and  to only the intended types.
  template<typename _Tp>
    struct __promote<_Tp, false>
    ;

  template<>
    struct __promote<long double>
    ;

  template<>
    struct __promote<double>
    ;

  template<>
    struct __promote<float>
    ;

  template<typename _Tp, typename _Up,
           typename _Tp2 = typename __promote<_Tp>::__type,
           typename _Up2 = typename __promote<_Up>::__type>
    struct __promote_2
    ;

  template<typename _Tp, typename _Up, typename _Vp,
           typename _Tp2 = typename __promote<_Tp>::__type,
           typename _Up2 = typename __promote<_Up>::__type,
           typename _Vp2 = typename __promote<_Vp>::__type>
    struct __promote_3
    ;

  template<typename _Tp, typename _Up, typename _Vp, typename _Wp,
           typename _Tp2 = typename __promote<_Tp>::__type,
           typename _Up2 = typename __promote<_Up>::__type,
           typename _Vp2 = typename __promote<_Vp>::__type,
           typename _Wp2 = typename __promote<_Wp>::__type>
    struct __promote_4
    ;

_GLIBCXX_END_NAMESPACE_VERSION
} // namespace

#endif 
 