 /* Definitions of status bits for `wait' et al.
   Copyright  1992-2014 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or  any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   .  */

#if !defined _SYS_WAIT_H && !defined _STDLIB_H
# error "Never include  directly; use  instead."
#endif


/* Everything extant so far uses these same bits.  */


/* If WIFEXITED, the low-order 8 bits of the status.  */
#define	__WEXITSTATUS	

/* If WIFSIGNALED, the terminating signal.  */
#define	__WTERMSIG	

/* If WIFSTOPPED, the signal that stopped the child.  */
#define	__WSTOPSIG	__WEXITSTATUS

/* Nonzero if STATUS indicates normal termination.  */
#define	__WIFEXITED	

/* Nonzero if STATUS indicates termination by a signal.  */
#define __WIFSIGNALED \
  

/* Nonzero if STATUS indicates the child is stopped.  */
#define	__WIFSTOPPED	

/* Nonzero if STATUS indicates the child continued after a stop.  We only
   define this if  provides the WCONTINUED flag bit.  */
#ifdef WCONTINUED
# define __WIFCONTINUED	
#endif

/* Nonzero if STATUS indicates the child dumped core.  */
#define	__WCOREDUMP	

/* Macros for constructing status values.  */
#define	__W_EXITCODE	
#define	__W_STOPCODE	
#define __W_CONTINUED		0xffff
#define	__WCOREFLAG		0x80


#ifdef	__USE_BSD

#pragma "/usr/include/endian.h"
#include "endian.h"

union wait
  {
    int w_status;
    struct
      {
# if	__BYTE_ORDER == __LITTLE_ENDIAN
	unsigned int __w_termsig:7; /* Terminating signal.  */
	unsigned int __w_coredump:1; /* Set if dumped core.  */
	unsigned int __w_retcode:8; /* Return code if exited normally.  */
	unsigned int:16;
# endif				/* Little endian.  */
# if	__BYTE_ORDER == __BIG_ENDIAN
	unsigned int:16;
	unsigned int __w_retcode:8;
	unsigned int __w_coredump:1;
	unsigned int __w_termsig:7;
# endif				/* Big endian.  */
      } __wait_terminated;
    struct
      {
# if	__BYTE_ORDER == __LITTLE_ENDIAN
	unsigned int __w_stopval:8; /* W_STOPPED if stopped.  */
	unsigned int __w_stopsig:8; /* Stopping signal.  */
	unsigned int:16;
# endif				/* Little endian.  */
# if	__BYTE_ORDER == __BIG_ENDIAN
	unsigned int:16;
	unsigned int __w_stopsig:8; /* Stopping signal.  */
	unsigned int __w_stopval:8; /* W_STOPPED if stopped.  */
# endif				/* Big endian.  */
      } __wait_stopped;
  };

# define w_termsig	__wait_terminated.__w_termsig
# define w_coredump	__wait_terminated.__w_coredump
# define w_retcode	__wait_terminated.__w_retcode
# define w_stopsig	__wait_stopped.__w_stopsig
# define w_stopval	__wait_stopped.__w_stopval

#endif	/* Use BSD.  */
 